@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/arc.c,v 1.2 2005/01/15 00:23:22 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/arc.c,v 1.9 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_ARC_C_
@@ -627,7 +627,7 @@ void DumpArcPath(FP, ObjPtr, xc, yc, xr, yr, dir, angle1, angle2,
       if (colorDump || !useGray) {
          if (preDumpSetup) PSUseColorPattern();
          fprintf(FP, "      %s\n", gPsCmd[PS_FLATTENPATH]);
-         DumpPatFill(FP, pen, 8, ObjPtr->bbox, 6, TRUE);
+         DumpPatFill(FP, pen, ObjPtr->bbox, 6, TRUE);
       } else {
          fprintf(FP, "      %s\n", gPsCmd[PS_STROKE]);
       }
@@ -714,7 +714,7 @@ void DumpArcObj(FP, ObjPtr)
          DumpArcPSPath(FP,xc,yc,xr,yr,dir,angle1,angle2,TRUE,"   ","      ");
          fprintf(FP, "   %s %s %s\n", gPsCmd[PS_CLOSEPATH],
                gPsCmd[PS_CLIP], gPsCmd[PS_NEWPATH]);
-         DumpPatFill(FP, fill, 8, ObjPtr->bbox, 3, TRUE);
+         DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
       } else {
          GrayCheck(fill);
          fprintf(FP, "   %s %s\n", GrayStr(fill), gPsCmd[PS_SETGRAY]);
@@ -1276,6 +1276,10 @@ struct ObjRec *CreateArcObj(xc, yc, x1, y1, x2, y2, dir, ltx, lty, w, h,
 
    obj_ptr->type = OBJ_ARC;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
@@ -2106,6 +2110,7 @@ void ReadArcObj(FP, Inbuf, ObjPtr)
 
    (*ObjPtr)->type = OBJ_ARC;
    (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
+   UtilStrCpyN((*ObjPtr)->color_str, sizeof((*ObjPtr)->color_str), color_str);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -2118,7 +2123,7 @@ void ReadArcObj(FP, Inbuf, ObjPtr)
       char inbuf[MAXSTRING+1];
       struct XfrmMtrxRec *ctm;
 
-      fgets(inbuf, MAXSTRING, FP);
+      (void)fgets(inbuf, MAXSTRING, FP);
       scanLineNum++;
       InitScan(inbuf, "\t\n, ");
 
