@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/attr.c,v 1.9 2006/02/10 23:21:18 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/attr.c,v 1.15 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -296,6 +296,7 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
    pStrSeg->style = curStyle;
    pStrSeg->sz_unit = GetCurSzUnit();
    pStrSeg->underline_on = curUnderlineOn;
+   pStrSeg->overline_on = curOverlineOn;
 
    pStrSeg->asc = canvasFontAsc;
    pStrSeg->des = canvasFontDes;
@@ -309,6 +310,8 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
    text_obj_ptr->y = ObjPtr->obbox.rby;
    text_obj_ptr->type = OBJ_TEXT;
    text_obj_ptr->color = ObjPtr->color;
+   memcpy(text_obj_ptr->color_str, ObjPtr->color_str,
+         sizeof(text_obj_ptr->color_str));
    text_obj_ptr->dirty = FALSE;
    text_obj_ptr->id = objId++;
    text_obj_ptr->rotation = 0;
@@ -2551,14 +2554,14 @@ int InitAttr()
          NULL) {
       maxAttrGroups = atoi(c_ptr);
    }
-#ifdef _TANGRAM_II
+   if (cmdLineTgrm2) {
 #ifdef DEFATTRGROUP
-   if (maxAttrGroups == 0) {
-      maxAttrGroups = 1;
-      attr_group_in_xdef = FALSE;
-   }
+      if (maxAttrGroups == 0) {
+         maxAttrGroups = 1;
+         attr_group_in_xdef = FALSE;
+      }
 #endif /* DEFATTRGROUP */
-#endif /* _TANGRAM_II */
+   }
    if (maxAttrGroups > 0) {
       gAttrGroupInfo = (AttrGroupInfo**)malloc(maxAttrGroups *
             sizeof(AttrGroupInfo*));
@@ -2579,8 +2582,7 @@ int InitAttr()
                      TOOL_NAME, buf);
                fprintf(stderr, "\n");
             }
-         } else {
-#ifdef _TANGRAM_II
+         } else if (cmdLineTgrm2) {
 #ifdef DEFATTRGROUP
             char *tmp_buf=UtilStrDup(DEFATTRGROUP);
 
@@ -2590,7 +2592,6 @@ int InitAttr()
             }
             UtilFree(tmp_buf);
 #endif /* DEFATTRGROUP */
-#endif /* _TANGRAM_II */
          }
       }
    } else if (maxAttrGroups == 0) {
