@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.15 2009/11/09 17:05:05 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.17 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -2029,7 +2029,7 @@ char *ReadString(Str)
    for (s=Str; *s != '\0'; s++) {
       if (*s == '"') {
          if (s[1] == '"') {
-            strcpy(s, s+1);
+            UtilStrCpy(s, s+1);
          } else {
             break;
          }
@@ -2037,7 +2037,7 @@ char *ReadString(Str)
          if (s[1] >= '0' && s[1] <= '3') {
             if (s[2] >= '0' && s[2] <= '7' && s[3] >= '0' && s[3] <= '7') {
                *s = (char)(((s[1]-'0')<<6)+((s[2]-'0')<<3)+(s[3]-'0'));
-               strcpy(s+1, s+4);
+               UtilStrCpy(s+1, s+4);
             } else {
                sprintf(gszMsgBox,
                      TgLoadString(STID_BAD_OCTAL_STRING_ENCOUNTERED),
@@ -2047,10 +2047,10 @@ char *ReadString(Str)
                } else {
                   Msg(gszMsgBox);
                }
-               strcpy(s, s+1);
+               UtilStrCpy(s, s+1);
             }
          } else {
-            strcpy(s, s+1);
+            UtilStrCpy(s, s+1);
          }
       }
    }
