@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/chat.c,v 1.5 2009/01/09 22:25:29 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/chat.c,v 1.8 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_CHAT_C_
@@ -366,7 +366,7 @@ void ChatEventHandler(input, handling_pti)
 
                switch (tdgt_notify.nf_arg) { /* Button[1-3] */
                case Button1:
-                  color_index = (int)TdgtBtnGetText(
+                  color_index = (int)(long)TdgtBtnGetText(
                         gstChatInfo.btn_ctl[CHAT_BTN_COLOR]);
                   if (++color_index >= maxColors) color_index = 0;
                   break;
@@ -374,13 +374,13 @@ void ChatEventHandler(input, handling_pti)
                   color_index = tdgt_notify.nf_arg2;
                   break;
                case Button3:
-                  color_index = (int)TdgtBtnGetText(
+                  color_index = (int)(long)TdgtBtnGetText(
                         gstChatInfo.btn_ctl[CHAT_BTN_COLOR]);
                   if (--color_index < 0) color_index = maxColors-1;
                   break;
                }
                TdgtBtnSetText(gstChatInfo.btn_ctl[CHAT_BTN_COLOR],
-                     (char*)color_index, NULL);
+                     (char*)(long)color_index, NULL);
                TdgtSmplEditSetColorIndex(gstChatInfo.edit_ctl, color_index);
                tgWriteProfileString("User", "Color",
                      colorMenuItems[color_index], gstChatInfo.ini_fname);
@@ -708,7 +708,7 @@ int CreateChatSubWindows()
          gstChatInfo.btn_win_info[CHAT_BTN_COLOR].w,
          gstChatInfo.btn_win_info[CHAT_BTN_COLOR].h,
          0, 0, TGMUTYPE_COLOR, TDGTBTN_CLICK, TGBS_NORMAL, INVALID,
-         (char*)color_index, &mosi);
+         (char*)(long)color_index, &mosi);
 
    gstChatInfo.edit_ctl = CreateTdgtSmplEdit(chatWindow, NULL, ID_CHAT_EDIT,
          gstChatInfo.edit_win_info.x, gstChatInfo.edit_win_info.y,
