@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmd.c,v 1.3 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmd.c,v 1.15 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_CMD_C_
@@ -86,7 +86,89 @@ void InsertCmd(PrevCmd, NextCmd, CmdPtr, ppFirstCmd, ppLastCmd)
    }
 }
 
-#ifdef _TGIF_WB2
+static
+char *GetCmdType(CmdType)
+   int CmdType;
+{
+   switch (CmdType) {
+   case CMD_COMPOSITE: return "CMD_COMPOSITE";
+   case CMD_NEW: return "CMD_NEW";
+   case CMD_DELETE: return "CMD_DELETE";
+   case CMD_MOVE: return "CMD_MOVE";
+   case CMD_STRETCH: return "CMD_STRETCH";
+   case CMD_ONE_TO_MANY: return "CMD_ONE_TO_MANY";
+   case CMD_MANY_TO_ONE: return "CMD_MANY_TO_ONE";
+   case CMD_REPLACE: return "CMD_REPLACE";
+   case CMD_GOTO_PAGE: return "CMD_GOTO_PAGE";
+   case CMD_WB_CLEARALL: return "CMD_WB_CLEARALL";
+   case CMD_CHAT_A_LINE: return "CMD_CHAT_A_LINE";
+   case CMD_WB_SLIDESHOW: return "CMD_WB_SLIDESHOW";
+   default: return "(unknown)";
+   }
+}
+
+static
+struct CmdRec *CreateCmdFromSerializationBuf(cmd_type, buf)
+   int cmd_type;
+   char *buf;
+{
+   struct CmdRec *cmd_ptr=NULL;
+
+   cmd_ptr = (struct CmdRec *)malloc(sizeof(struct CmdRec));
+   if (cmd_ptr == NULL) FailAllocMessage(); 
+   memset(cmd_ptr, 0, sizeof(struct CmdRec)); 
+
+   cmd_ptr->type = cmd_type;
+   cmd_ptr->serialization_buf = UtilStrDup(buf);
+   if (cmd_ptr->serialization_buf == NULL) FailAllocMessage();
+
+   return cmd_ptr;
+}
+
+void CopyAndInsertCmd(from_remote, serialization_buf, PrevCmd, NextCmd, CmdPtr,
+      ppFirstCmd, ppLastCmd)
+   int from_remote;
+   char *serialization_buf;
+   struct CmdRec *PrevCmd, *NextCmd, *CmdPtr;
+   struct CmdRec **ppFirstCmd, **ppLastCmd;
+   /* add CmdPtr between PrevCmd and NextCmd */
+   /* update *ppFirstCmd and *ppLastCmd */
+{
+   if ((!from_remote && CmdPtr->undone) || (from_remote && !CmdPtr->undone)) {
+      sprintf(gszMsgBox, "(%s) Found an %s command, type = %s.",
+            (from_remote ? "from_remote" : "local"),
+            (CmdPtr->undone ? "undone" : "not undone"),
+            GetCmdType(CmdPtr->type));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+   } else {
+      struct CmdRec *cmd_ptr=NULL;
+
+#ifdef _TGIF_DBG_WB2
+      fprintf(stdout,
+            "<*><*><*>(%s) CopyAndInsertCmd() a command of type: %s.\n",
+            (from_remote ? "from_remote" : "local"),
+            GetCmdType(CmdPtr->type));
+      fflush(stdout);
+#endif /* _TGIF_DBG_WB2 */
+
+      cmd_ptr = CreateCmdFromSerializationBuf(CmdPtr->type, serialization_buf);
+
+      cmd_ptr->prev = PrevCmd;
+      cmd_ptr->next = NextCmd;
+
+      if (PrevCmd == NULL) {
+         (*ppFirstCmd) = cmd_ptr;
+      } else {
+         PrevCmd->next = cmd_ptr;
+      }
+      if (NextCmd == NULL) {
+         (*ppLastCmd) = cmd_ptr;
+      } else {
+         NextCmd->prev = cmd_ptr;
+      }
+   }
+}
+
 static
 void UnlinkCmd(CmdPtr, ppFirstCmd, ppLastCmd)
    struct CmdRec *CmdPtr;
@@ -103,7 +185,6 @@ void UnlinkCmd(CmdPtr, ppFirstCmd, ppLastCmd)
       CmdPtr->next->prev = CmdPtr->prev;
    }
 }
-#endif /* _TGIF_WB2 */
 
 void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
    struct CmdRec *CmdPtr;
@@ -246,10 +327,14 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
       if (CmdPtr->subcmd != NULL) free(CmdPtr->subcmd);
       break;
    }
-#ifdef _TGIF_WB2
-   if (CmdPtr->sender_process_id != NULL) UtilFree(CmdPtr->sender_process_id);
-#endif /* _TGIF_WB2 */
-
+   if (gstWBInfo.do_whiteboard) {
+      if (CmdPtr->sender_process_id != NULL) {
+         UtilFree(CmdPtr->sender_process_id);
+      }
+      if (CmdPtr->serialization_buf != NULL) {
+         UtilFree(CmdPtr->serialization_buf);
+      }
+   }
    free(CmdPtr);
 }
 
@@ -403,12 +488,12 @@ int ObjHasFullID(obj_ptr, psz_full_id)
    char *psz_full_id;
 {
    if (*psz_full_id != '#') {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
-            "Unexpected *psz_full_id != '#' in ObjHasFullID()",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
+               "Unexpected *psz_full_id != '#' in ObjHasFullID()",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    psz_full_id = strchr(psz_full_id, ':');
    psz_full_id++;
@@ -429,24 +514,24 @@ int GetPositionFromFullID(psz_full_id)
    char *psz=NULL;
 
    if (*psz_full_id != '#') {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
-            "Unexpected *psz_full_id != '#' in GetPositionFromFullID()",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
+               "Unexpected *psz_full_id != '#' in GetPositionFromFullID()",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    psz = strchr(++psz_full_id, ':');
    *psz = '\0';
    if (sscanf(psz_full_id, "%d", &pos_to_match) != 1 ||
          pos_to_match == INVALID) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
-            "Format error in parsing pos_to_match in",
-            "GetPositionFromFullID()", "cannot get pos_to_match",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
+               "Format error in parsing pos_to_match in",
+               "GetPositionFromFullID()", "cannot get pos_to_match",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    *psz = ':';
 
@@ -569,13 +654,13 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          if (!modify_before_pointers) {
             if (sscanf(psz_full_id, "%d", &pos_to_match) != 1 ||
                   pos_to_match == INVALID) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-               sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
-                     "Format error in parsing pos_to_match in",
-                     "SetupBeforePointers()", "cannot get pos_to_match",
-                     psz_full_id, gstWBInfo.pid);
-               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+               if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+                  sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
+                        "Format error in parsing pos_to_match in",
+                        "SetupBeforePointers()", "cannot get pos_to_match",
+                        psz_full_id, gstWBInfo.pid);
+                  MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+               }
             }
          }
          *psz++ = ':';
@@ -584,12 +669,12 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
       psz = strchr(psz_full_id, '/');
       *pid_and_host = '\0';
       if (psz == NULL) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-         sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
-               "Format error in parsing id in SetupBeforePointers()",
-               "cannot find ':'", psz_full_id, gstWBInfo.pid);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+         if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+            sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
+                  "Format error in parsing id in SetupBeforePointers()",
+                  "cannot find ':'", psz_full_id, gstWBInfo.pid);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
          ok = FALSE;
          break;
       } else {
@@ -600,12 +685,12 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
                obj_is_tgifobj = TRUE;
             }
          } else {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-            sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
-                  "Format error in parsing id in SetupBeforePointers()",
-                  "cannot find id", psz_full_id, gstWBInfo.pid);
-            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+            if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+               sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
+                     "Format error in parsing id in SetupBeforePointers()",
+                     "cannot find id", psz_full_id, gstWBInfo.pid);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            }
             ok = FALSE;
             break;
          }
@@ -645,25 +730,25 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          }
       }
       if (ok && !found) {
-#ifdef _TGIF_WB2
+         if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
-         fprintf(stderr, "%s: '%s' %s, pid = %ld.\n\t(%s=%1d, %s='%s')\n",
-               "@*@ Cannot find object id", psz_full_id,
-               "in SetupBeforePointers()", gstWBInfo.pid, "id", id,
-               "pid_and_host", pid_and_host);
+            fprintf(stderr, "%s: '%s' %s, pid = %ld.\n\t(%s=%1d, %s='%s')\n",
+                  "@*@ Cannot find object id", psz_full_id,
+                  "in SetupBeforePointers()", gstWBInfo.pid, "id", id,
+                  "pid_and_host", pid_and_host);
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
+         }
          ok = FALSE;
          break;
       }
    }
    if (ok && index != CmdPtr->count_before) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s (%1d) %s (%1d) %s, pid = %ld.",
-            "Index", index, "!= CmdPtr->count_before", CmdPtr->count_before,
-            "in SetupBeforePointers()", gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s (%1d) %s (%1d) %s, pid = %ld.",
+               "Index", index, "!= CmdPtr->count_before", CmdPtr->count_before,
+               "in SetupBeforePointers()", gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
       ok = FALSE;
    }
    if (!ok) {
@@ -797,7 +882,7 @@ int UndoOrRedoMoveCmd(CmdPtr)
          for (obj_ptr=topObj; obj_ptr != NULL; obj_ptr=next_obj) {
             next_obj = obj_ptr->next;
             if (ObjHasFullID(obj_ptr,
-                  (char*)(int)(CmdPtr->pos_before[count]))) {
+                  (char*)(long)(CmdPtr->pos_before[count]))) {
                count++;
                sel_ptr->obj = obj_ptr;
                if (!obj_ptr->locked) MoveObj(obj_ptr, dx, dy);
@@ -871,9 +956,8 @@ void UpdateXPmObjectsInACmd(CmdPtr)
 }
 
 static
-int UpdatePixelInACmd(CmdPtr, old_color_str)
+int UpdatePixelInACmd(CmdPtr)
    struct CmdRec *CmdPtr;
-   char **old_color_str;
 {
    struct CmdRec *cmd_ptr;
    struct SelRec *sel_ptr;
@@ -882,12 +966,16 @@ int UpdatePixelInACmd(CmdPtr, old_color_str)
    switch (CmdPtr->type) {
    case CMD_COMPOSITE:
       for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
-         if (UpdatePixelInACmd(cmd_ptr, old_color_str)) changed = TRUE;
+         if (UpdatePixelInACmd(cmd_ptr)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_NEW:
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_DELETE: break;
@@ -895,16 +983,22 @@ int UpdatePixelInACmd(CmdPtr, old_color_str)
    case CMD_STRETCH: break;
    case CMD_REPLACE:
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_ONE_TO_MANY: break;
    case CMD_MANY_TO_ONE:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_GOTO_PAGE: break;
@@ -920,8 +1014,7 @@ int RefreshColormap(nRedoing, CmdPtr)
     * This function mimics FlushColormap() in color.c.
     */
 {
-   int i=0, old_allocated_colors, changed=FALSE, saved_color_layers;
-   char **old_color_str=NULL;
+   int changed=FALSE, saved_color_layers;
    struct ObjRec *obj_ptr=NULL;
    struct PageRec *page_ptr=NULL;
    Colormap colormap=XCopyColormapAndFree(mainDisplay, mainColormap);
@@ -949,15 +1042,6 @@ int RefreshColormap(nRedoing, CmdPtr)
          }
       }
    }
-   old_color_str = (char**)malloc(maxColors*sizeof(char*));
-   if (old_color_str == NULL) FailAllocMessage();
-   old_allocated_colors = maxColors;
-   for (i = 0; i < maxColors; i++) {
-      old_color_str[i] =
-            (char*)malloc((strlen(colorMenuItems[i])+1)*sizeof(char));
-      if (old_color_str[i] == NULL) FailAllocMessage();
-      strcpy(old_color_str[i], colorMenuItems[i]);
-   }
    initColorDontReload = TRUE;
    CleanUpColors();
    XFreeColormap(mainDisplay, mainColormap);
@@ -979,28 +1063,26 @@ int RefreshColormap(nRedoing, CmdPtr)
 
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
       }
       for (cmd_ptr=CmdPtr->next; cmd_ptr != NULL; cmd_ptr=cmd_ptr->next) {
-         if (UpdatePixelInACmd(cmd_ptr, old_color_str)) {
+         if (UpdatePixelInACmd(cmd_ptr)) {
             changed = TRUE;
          }
       }
    } else {
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
       }
    }
    RestoreStatusStrings();
-   for (i = 0; i < old_allocated_colors; i++) free(old_color_str[i]);
-   free(old_color_str);
 
    DestroyPinnedMenu(MENU_COLOR);
    if (colorLayers) {
@@ -1022,14 +1104,12 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
    int need_clear_and_redraw=FALSE;
    int *actual_pos_before=NULL;
 
-#ifdef _TGIF_WB2
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
    if (gstWBInfo.do_whiteboard) {
       fprintf(stderr, "@@@ Beginning of UndoOrRedoReplaceCmd()...\n");
       fprintf(stderr, "\tCmdPtr->serialized=%1d...\n", CmdPtr->serialized);
    }
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
    if (CmdPtr->serialized) {
       int shuffle=FALSE;
 
@@ -1037,13 +1117,11 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
          CmdPtr->first_redo_after_deserialize = FALSE;
          shuffle = TRUE;
       }
-#ifdef _TGIF_WB2
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
       if (gstWBInfo.do_whiteboard) {
          fprintf(stderr, "In UndoOrRedoReplaceCmd(), shuffle=%1d.\n", shuffle);
       }
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
       if (CmdPtr->include_tgif_obj) {
          AddObj(NULL, topObj, tgifObj);
       }
@@ -1261,7 +1339,7 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
       pos_table = (CmdPtr->undone) ? CmdPtr->pos_before : CmdPtr->pos_after;
       max_count = (CmdPtr->undone) ? CmdPtr->count_before : CmdPtr->count_after;
 
-      if (CmdPtr->serialized && !CmdPtr->undone) {
+      if (CmdPtr->serialized && CmdPtr->undone) {
          pos_table = actual_pos_before;
       }
       pos = count = 0;
@@ -1305,7 +1383,7 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
 
    pos = count = 0;
    sel_ptr = topSel;
-   if (CmdPtr->serialized && !CmdPtr->undone) {
+   if (CmdPtr->serialized && CmdPtr->undone) {
       pos_table = actual_pos_before;
    }
    for (obj_ptr=topObj; obj_ptr!=NULL; obj_ptr=next_obj, pos++) {
@@ -1852,7 +1930,6 @@ void FreeAfterSel(CmdPtr)
    CmdPtr->top_after = CmdPtr->bot_after = NULL;
 }
 
-#ifdef _TGIF_WB2
 static
 void PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
       NumObjs, psi)
@@ -1877,7 +1954,6 @@ void PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
    psi->count_before = stackingCount;
    psi->pos_before_has_ids = stackingPositionHasIds;
 }
-#endif /* _TGIF_WB2 */
 
 void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    int CmdType, NumObjs;
@@ -1885,14 +1961,10 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    struct SelRec *TopSel, *BotSel;
 {
    int logical_clock=0;
-#ifdef _TGIF_WB2
-   int wb_send_data_failed=FALSE;
-#endif /* _TGIF_WB2 */
+   int wb_send_data_failed=FALSE, plain_wb_data_sz=0;
+   char *psz_wb_data=NULL, *psz_plain_wb_data=NULL;
 
    if (gstWBInfo.do_whiteboard) {
-#ifdef _TGIF_WB2
-      char *psz_wb_data=NULL;
-
       if (gstWBInfo.dont_serialize || CmdType == CMD_GOTO_PAGE) {
          /*
           * One of the important thing that happens in SerializeCmd()
@@ -1912,15 +1984,16 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
          
          serializingFile = TRUE;
 
-         if ((psz_wb_data=SerializeCmd(CmdType, &si, &logical_clock)) != NULL) {
+         psz_plain_wb_data = NULL;
+         plain_wb_data_sz = 0;
+         if ((psz_wb_data=SerializeCmd(CmdType, &si, &logical_clock,
+               &psz_plain_wb_data, &plain_wb_data_sz)) != NULL) {
             if (!SendWBData(psz_wb_data,logical_clock)) {
                wb_send_data_failed = TRUE;
             }
-            SerializeFreeBuf(psz_wb_data);
          }
          serializingFile = FALSE;
       }
-#endif /* _TGIF_WB2 */
    } else {
       if (historyDepth == 0) return;
 
@@ -1972,6 +2045,28 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
       curCmd->sender_process_id = UtilStrDup(gszLocalPID);
       if (curCmd->sender_process_id == NULL) FailAllocMessage();
 
+      if (!wb_send_data_failed) {
+         /*
+          * FindShadowCmdInsertionPoint() examine the logical clock of the new
+          *       command and decides where to insert it in the shadow cmd list.
+          */
+         FindCmdInsertionPoint(curCmd, &immed_right_cmd);
+
+         if (immed_right_cmd == NULL) {
+            /* append */
+            CopyAndInsertCmd(FALSE,
+                  (psz_plain_wb_data==NULL ? psz_wb_data : psz_plain_wb_data),
+                  gstWBInfo.last_shadow_cmd, NULL, curCmd,
+                  &gstWBInfo.first_shadow_cmd, &gstWBInfo.last_shadow_cmd);
+         } else {
+            /* insert */
+            CopyAndInsertCmd(FALSE,
+                  (psz_plain_wb_data==NULL ? psz_wb_data : psz_plain_wb_data),
+                  immed_right_cmd->prev, immed_right_cmd, curCmd,
+                  &gstWBInfo.first_shadow_cmd, &gstWBInfo.last_shadow_cmd);
+         }
+      }
+      immed_right_cmd = NULL;
       /*
        * FindCmdInsertionPoint() examine the logical clock of the new command
        *       and decides where to insert it.
@@ -2028,14 +2123,19 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
             SubCmdPtr->detail.slideshow.into_slideshow;
       break;
    }
-#ifdef _TGIF_WB2
-   if (wb_send_data_failed) {
-      UndoACmd(curCmd, FALSE, TRUE);
-      UnlinkCmd(curCmd, &gstWBInfo.first_cmd, &gstWBInfo.last_cmd);
-      DeleteARedoRecord(curCmd, (-1.0), (-1.0));
+   if (gstWBInfo.do_whiteboard) {
+      if (wb_send_data_failed) {
+         UndoACmd(curCmd, FALSE, TRUE);
+         UnlinkCmd(curCmd, &gstWBInfo.first_cmd, &gstWBInfo.last_cmd);
+         DeleteARedoRecord(curCmd, (-1.0), (-1.0));
+      }
+      if (psz_wb_data != NULL) {
+         SerializeFreeBuf(psz_wb_data);
+      }
+      if (psz_plain_wb_data != NULL) {
+         UtilFree(psz_plain_wb_data);
+      }
    }
-#endif /* _TGIF_WB2 */
-
    curCmd = lastCmd;
 }
 
