@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.3 2005/01/15 00:23:22 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.35 2011/06/09 15:55:15 cvsps Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -410,7 +410,48 @@
 #define CMDID_IMPORTPBMFILE           468
 #define CMDID_IMPORTPGMFILE           469
 #define CMDID_EDITATTRGROUPINEDITOR   470
+#define CMDID_LINKEXTJPEGFILE         471
+#define CMDID_NEXTSLIDE               472
+#define CMDID_PREVSLIDE               473
+#define CMDID_SETOBJSHADOWCOLOR       474
+#define CMDID_SETOBJSHADOWOFFSETS     475
+#define CMDID_ADDOBJSHADOW            476
+#define CMDID_REMOVEOBJSHADOW         477
+#define CMDID_COPYUTF8                478
+#define CMDID_PASTEUTF8               479
+#define CMDID_REDUCETOMOBILEWEBCOLORS 480
+#define CMDID_CREATEPIXMAPFROMSEL     481
+#define CMDID_TOGGLEROTATEPIVOT       482
+#define CMDID_SPECIFYROTATEPIVOT      483
+#define CMDID_RESETROTATEPIVOT        484
+#define CMDID_NEXTPOLYROTATEPIVOT     485
+#define CMDID_MOVEROTATEPIVOTARCCNTR  486
+#define CMDID_MOVEROTATEPIVOTCENTER   487
+#define CMDID_MOVEROTATEPIVOTLT       488
+#define CMDID_MOVEROTATEPIVOTRT       489
+#define CMDID_MOVEROTATEPIVOTLB       490
+#define CMDID_MOVEROTATEPIVOTRB       491
+#define CMDID_MOVEROTATEPIVOTLEFT     492
+#define CMDID_MOVEROTATEPIVOTRIGHT    493
+#define CMDID_MOVEROTATEPIVOTTOP      494
+#define CMDID_MOVEROTATEPIVOTBOTTOM   495
+#define CMDID_CHOOSEROTATEPIVOT       496
+#define CMDID_COLORWHEEL              497
+#define CMDID_EXTENDSEGMENT           498
+#define CMDID_TOGGLETIGHTERSPLINES    499
 
-#define MAXCMDIDS 471
+#define CMDID_INSERTSYMBOL                  500
+#define CMDID_TOGGLERIGHTMARGIN             501
+#define CMDID_SPECIFYRIGHTMARGIN            502
+#define CMDID_CHANGEALLSELFONTOVERLINE      503
+#define CMDID_TOGGLEFLOODREPLACECOLORTHRESH 504
+#define CMDID_SETFLOODREPLACECOLORTHRESH    505
+#define CMDID_REMOVETRANSPIXEL              506
+#define CMDID_REPLACECOLORWITHTRANS         507
+#define CMDID_NEWRECTFROMBOUNDINGBOX        508
+#define CMDID_NEWOVALFROMBOUNDINGBOX        509
+#define CMDID_NEWRCBOXFROMBOUNDINGBOX       510
+
+#define MAXCMDIDS 511
 
 #endif /*_CMDIDS_H_*/
