@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.34 2011/05/16 16:21:56 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.35 2011/06/09 15:55:15 cvsps Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -461,7 +461,10 @@
 #define CMDID_SETFLOODREPLACECOLORTHRESH    505
 #define CMDID_REMOVETRANSPIXEL              506
 #define CMDID_REPLACECOLORWITHTRANS         507
+#define CMDID_NEWRECTFROMBOUNDINGBOX        508
+#define CMDID_NEWOVALFROMBOUNDINGBOX        509
+#define CMDID_NEWRCBOXFROMBOUNDINGBOX       510
 
-#define MAXCMDIDS 508
+#define MAXCMDIDS 511
 
 #endif /*_CMDIDS_H_*/
