@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.7 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.25 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -49,6 +49,7 @@
 #include "stk.e"
 #include "strtbl.e"
 #include "text.e"
+#include "tgcwheel.e"
 #include "util.e"
 #include "xpixmap.e"
 
@@ -62,6 +63,7 @@ int	colorIndex=0;
 int	*colorPixels=NULL;
 int	*xorColorPixels=NULL;
 int	*colorLayerOn=NULL;
+char	defaultColorStr[COLORSTRLEN];
 char	myFgColorStr[COLORSTRLEN];
 char	myBgColorStr[COLORSTRLEN];
 char	myRubberBandColorStr[COLORSTRLEN];
@@ -87,11 +89,14 @@ char	altEditTextHighlightColorStr[COLORSTRLEN];
 int	altEditTextHighlightIndex=INVALID;
 int	useAltEditTextBgColor=FALSE;
 
+char	pngExportTransparentColor[MAXSTRING];
+
 static int colorWindowFirstIndex=0;
 static int colorItemH=12;
 
 static int	canChangeAttrColor=FALSE;
 static int	defaultColorIndexInXDefaults=FALSE;
+static int	defaultColorInXDefaults=FALSE;
 static int	allocatedMaxColors=MAXCOLORS;
 
 static char *defaultColorMenuItems[MAXCOLORS]={
@@ -197,7 +202,6 @@ int TgifParseColor(psz_color, p_color)
    return (int)XParseColor(mainDisplay, mainColormap, psz_color, p_color);
 }
 
-static
 int ParseAndAllocColorByName(colorname, color, red_req, green_req, blue_req)
    char *colorname;
    XColor *color;
@@ -442,14 +446,180 @@ int DoSetDefaultBgColor()
    return TRUE;
 }
 
+static int std_palette8[] = { 0x00, 0xff };
+static int std_palette27[] = { 0x00, 0x80, 0xff };
+static int std_palette64[] = { 0x00, 0x55, 0xaa, 0xff };
+static int std_palette216[] = { 0x00, 0x33, 0x66, 0x99, 0xcc, 0xff };
+static char *openoffice_galaxy_palette[] = {
+      /* black */
+      "#000000",
+      "#151a1f",
+      "#29333c",
+      "#4b5a66",
+      "#8b97a0",
+      "#bec5c9",
+      "#d8dcdf",
+      "#edf1f5",
+      "#ffffff",
+      /* blue-gray */
+      "#123349",
+      "#35556b",
+      "#5382a1",
+      "#a3b8cb",
+      "#dae3ea",
+      /* more-blue-gray */
+      "#003e6d",
+      "#005a9b",
+      "#0877bc",
+      "#71c0eb",
+      "#a4cef0",
+      "#ddedfa",
+      /* blue */
+      "#063d80",
+      "#0646a2",
+      "#0852d4",
+      "#085dff",
+      "#0fa0ff",
+      "#b8f1ff",
+      /* green */
+      "#063e00",
+      "#096000",
+      "#128902",
+      "#1cbb04",
+      "#8dff1a",
+      "#cfff9e",
+      /* yellow */
+      "#b2bc00",
+      "#7f7800",
+      "#46580b",
+      "#314005",
+      "#634900",
+      "#c69200",
+      /* orange */
+      "#f26a00",
+      "#faa217",
+      "#ffc726",
+      "#ffdf24",
+      "#fff67b",
+      "#ffff9e",
+      /* red */
+      "#6d0000",
+      "#8e0000",
+      "#c70909",
+      "#ff0000",
+      "#fc5b5b",
+      "#ffa199",
+      /* purple */
+      "#2e008f",
+      "#8866cc",
+      "#c2b0e5",
+      NULL
+};
+static char *openoffice_google_palette[] = {
+      /* black */
+      "#000000",
+      "#262626",
+      "#404040",
+      "#575757",
+      "#747474",
+      "#909090",
+      "#a8a8a8",
+      "#c5c5c5",
+      "#e6e6e6",
+      "#ffffff",
+      /* saturated */
+      "#ff0000",
+      "#ff9900",
+      "#ffff00",
+      "#98fe00",
+      "#00ff00",
+      "#00ffff",
+      "#0035ff",
+      "#9900ff",
+      "#ff00ff",
+      "#ff0187",
+      /* red */
+      "#660000",
+      "#990000",
+      "#cc0000",
+      "#f03939",
+      "#ea9999",
+      "#f4cccc",
+      /* orange */
+      "#783f04",
+      "#b45f06",
+      "#e69138",
+      "#f6b26b",
+      "#f9cb9c",
+      "#fce5cd",
+      /* yellow */
+      "#7f6b00",
+      "#bfa000",
+      "#f1d932",
+      "#ffe766",
+      "#ffec99",
+      "#fff6cc",
+      /* light green */
+      "#2c4a00",
+      "#4c7f00",
+      "#5f9f00",
+      "#84be2d",
+      "#acdb65",
+      "#d1e7b1",
+      /* green */
+      "#1c4e13",
+      "#26761d",
+      "#5ba84f",
+      "#84c47d",
+      "#b0d7a8",
+      "#d6ead3",
+      /* cyan */
+      "#0c343d",
+      "#134f5c",
+      "#45818e",
+      "#76a5af",
+      "#a2c4c9",
+      "#d0e0e3",
+      /* blue */
+      "#073763",
+      "#0b5394",
+      "#3d85c6",
+      "#6fa8dc",
+      "#9fc5e8",
+      "#cfe2f3",
+      /* purple */
+      "#20124d",
+      "#351c75",
+      "#674ea7",
+      "#8e7cc3",
+      "#b4a7d6",
+      "#d9d2e9",
+      /* magenta */
+      "#4c114a",
+      "#741b68",
+      "#a64d9b",
+      "#c27bbd",
+      "#d5a6d4",
+      "#ead1ea",
+      /* redish */
+      "#5a002f",
+      "#94004e",
+      "#c10066",
+      "#e53b94",
+      "#e895c1",
+      "#f6d1e5",
+      NULL
+};
+
 void InitColor()
 {
-   int i, index, looking_for_default_color, tmp_max, color_from_pixmap=FALSE;
+   int i, index=0, looking_for_default_color=FALSE, tmp_max, color_from_pixmap=FALSE;
    XColor color;
    char buf[80], *c_ptr, fg_color[80], bg_color[80], brdr_color[80];
    long bg_gray=(long)0;
    int bg_allocated=FALSE, fg_allocated=FALSE, brdr_allocated=FALSE, ncolors=0;
-   char **color_str=NULL;
+   int std_palette=0, need_to_handle_additional_colors=FALSE;
+   char **color_str=NULL, *additional_colors_buf=NULL;
 
    InitLocalRGBTxt();
 
@@ -473,29 +643,79 @@ void InitColor()
          colorDump = FALSE;
       }
    }
+   defaultColorInXDefaults = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColor"))
+         != NULL) {
+      defaultColorInXDefaults = TRUE;
+      UtilStrCpyN(defaultColorStr, sizeof(defaultColorStr), c_ptr);
+   }
    defaultColorIndexInXDefaults = FALSE;
    defaultColorIndex = 9;
-   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColorIndex"))
-         != NULL) {
-      defaultColorIndexInXDefaults = TRUE;
-      defaultColorIndex = atoi(c_ptr);
+   if (!defaultColorInXDefaults) {
+      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColorIndex"))
+            != NULL) {
+         defaultColorIndexInXDefaults = TRUE;
+         defaultColorIndex = atoi(c_ptr);
+      }
    }
    color_from_pixmap = InitColorFromXPixmap(&ncolors, &color_str);
    if (color_from_pixmap && ncolors > 0 && color_str != NULL) {
       maxColors = ncolors;
+      need_to_handle_additional_colors = TRUE;
    } else {
-      maxColors = MAXCOLORS;
-      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "MaxColors")) != NULL) {
-         maxColors = atoi(c_ptr);
-         if (maxColors <= 0) {
-            fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
-                  TOOL_NAME, "MaxColors", c_ptr, MAXCOLORS);
-            fprintf(stderr, "\n");
-            maxColors = MAXCOLORS;
+      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseMobileWebSafePalette")) != NULL) {
+         maxColors = std_palette = 216;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette216")) != NULL) {
+         maxColors = std_palette = 216;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette64")) != NULL) {
+         maxColors = std_palette = 64;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette27")) != NULL) {
+         maxColors = std_palette = 27;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette8")) != NULL) {
+         maxColors = std_palette = 8;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseOpenOfficeGalaxyPalette")) != NULL) {
+         maxColors = std_palette = 53;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseOpenOfficeGooglePalette")) != NULL) {
+         maxColors = std_palette = 80;
+      } else {
+         maxColors = MAXCOLORS;
+         if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "MaxColors")) != NULL) {
+            maxColors = atoi(c_ptr);
+            if (maxColors <= 0) {
+               fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+                     TOOL_NAME, "MaxColors", c_ptr, MAXCOLORS);
+               fprintf(stderr, "\n");
+               maxColors = MAXCOLORS;
+            }
          }
       }
+      if (std_palette > 0) {
+         need_to_handle_additional_colors = TRUE;
+      }
    }
+   if (need_to_handle_additional_colors) {
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"AdditionalColors")) !=
+            NULL) {
+         char *buf_copy=UtilStrDup(c_ptr), *psz_state=NULL;
 
+         if (buf_copy == NULL) FailAllocMessage();
+         UtilTrimBlanks(buf_copy);
+         additional_colors_buf = UtilStrDup(buf_copy);
+         if (additional_colors_buf == NULL) FailAllocMessage();
+         for (c_ptr=UtilStrTok(buf_copy, ",\t\n\r", &psz_state); c_ptr != NULL;
+               c_ptr=UtilStrTok(NULL, ",\t\n\r", &psz_state)) {
+            maxColors++;
+         }
+         UtilFree(buf_copy);
+      }
+   }
    if (colorDisplay) {
       if (((cmdLineForeground != NULL && cmdLineBackground == NULL) ||
             (cmdLineForeground == NULL && cmdLineBackground != NULL)) &&
@@ -584,24 +804,146 @@ void InitColor()
       free(color_str);
       color_str = NULL;
    } else {
-      for (i=0; i < maxColors; i++) {
-         sprintf(buf, "Color%1d", i);
-         if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, buf)) != NULL) {
-            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
-         } else if (i < MAXCOLORS) {
-            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN,
-                  defaultColorMenuItems[i]);
-         } else {
-            fprintf(stderr, TgLoadString(STID_CANNOT_GET_X_DEFAULT),
-                  TOOL_NAME, buf);
-            fprintf(stderr, "\n");
-            exit(-1);
+      int r=0, g=0, b=0;
+
+      switch (std_palette) {
+      case 0:
+         for (i=0; i < maxColors; i++) {
+            sprintf(buf, "Color%1d", i);
+            if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, buf)) != NULL) {
+               UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
+            } else if (i < MAXCOLORS) {
+               UtilStrCpyN(colorMenuItems[i], COLORSTRLEN,
+                     defaultColorMenuItems[i]);
+            } else {
+               fprintf(stderr, TgLoadString(STID_CANNOT_GET_X_DEFAULT),
+                     TOOL_NAME, buf);
+               fprintf(stderr, "\n");
+               exit(-1);
+            }
+            UtilTrimBlanks(colorMenuItems[i]);
+            colorLayerOn[i] = TRUE;
          }
-         UtilTrimBlanks(colorMenuItems[i]);
-         colorLayerOn[i] = TRUE;
+         break;
+      case 8:
+         i = 0;
+         for (r=0; r < 2; r++) {
+            for (g=0; g < 2; g++) {
+               for (b=0; b < 2; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette8[r], std_palette8[g], std_palette8[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 7;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 27:
+         i = 0;
+         for (r=0; r < 3; r++) {
+            for (g=0; g < 3; g++) {
+               for (b=0; b < 3; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette27[r], std_palette27[g], std_palette27[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 26;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 64:
+         i = 0;
+         for (r=0; r < 4; r++) {
+            for (g=0; g < 4; g++) {
+               for (b=0; b < 4; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette64[r], std_palette64[g], std_palette64[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 63;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 216:
+         i = 0;
+         for (r=0; r < 6; r++) {
+            for (g=0; g < 6; g++) {
+               for (b=0; b < 6; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette216[r], std_palette216[g], std_palette216[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 215;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 53: /* OpenOffice Galaxy Palette */
+         for (i=0; i < std_palette; i++) {
+            strcpy(colorMenuItems[i], openoffice_galaxy_palette[i]);
+            colorLayerOn[i] = TRUE;
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 8;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 80: /* OpenOffice Google Palette */
+         for (i=0; i < std_palette; i++) {
+            strcpy(colorMenuItems[i], openoffice_google_palette[i]);
+            colorLayerOn[i] = TRUE;
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 9;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
       }
    }
+   if (need_to_handle_additional_colors && additional_colors_buf != NULL) {
+      char *psz_state=NULL;
 
+      for (c_ptr=UtilStrTok(additional_colors_buf, ",\t\n\r", &psz_state); c_ptr != NULL;
+            c_ptr=UtilStrTok(NULL, ",\t\n\r", &psz_state)) {
+         UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
+         colorLayerOn[i] = TRUE;
+         i++;
+      }
+      UtilFree(additional_colors_buf);
+   }
    if (threeDLook) {
       static int initVryLtPixel=FALSE;
 
@@ -716,7 +1058,16 @@ void InitColor()
          exit (-1);
       }
       looking_for_default_color = FALSE;
-      if (defaultColorIndexInXDefaults) {
+      if (defaultColorInXDefaults) {
+         looking_for_default_color = TRUE;
+         for (i=0; i < maxColors; i++) {
+            if (UtilStrICmp(defaultColorStr, colorMenuItems[i]) == 0) {
+               defaultColorIndex = i;
+               looking_for_default_color = FALSE;
+               break;
+            }
+         }
+      } else if (defaultColorIndexInXDefaults) {
          if (defaultColorIndexInXDefaults && defaultColorIndex >= maxColors) {
             fprintf(stderr, TgLoadString(STID_DEFCOLORINDEX_GE_MAXCOLORS),
                   TOOL_NAME, "DefaultColorIndex", TOOL_NAME, "MaxColors",
@@ -762,6 +1113,35 @@ void InitColor()
                   ((long)color.blue);
          }
       }
+      if (defaultColorInXDefaults && looking_for_default_color) {
+         if (!ParseAndAllocColorByName(defaultColorStr, &color, NULL, NULL, NULL)) {
+            fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+                  TOOL_NAME, "DefaultColor", defaultColorStr);
+            fprintf(stderr, "\n");
+            defaultColorInXDefaults = FALSE;
+         }
+         for (i=0; i < maxColors; i++) {
+            if (color.pixel == colorPixels[i]) {
+               defaultColorIndex = i;
+               looking_for_default_color = FALSE;
+               break;
+            }
+         }
+         if (looking_for_default_color) {
+            int new_alloc=FALSE;
+
+            index = QuickFindColorIndex(NULL, defaultColorStr, &new_alloc, FALSE);
+            if (index == INVALID) {
+               fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+                     TOOL_NAME, "DefaultColor", defaultColorStr);
+               fprintf(stderr, "\n");
+               defaultColorInXDefaults = FALSE;
+            } else {
+               defaultColorIndex = index;
+               looking_for_default_color = FALSE;
+            }
+         }
+      }
       if (looking_for_default_color) {
          long val, val1;
 
@@ -936,6 +1316,23 @@ void InitColor()
          colorIndex = color_index;
       }
    }
+   UtilStrCpyN(pngExportTransparentColor, sizeof(pngExportTransparentColor),
+         myBgColorStr);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PNGExportTransparentColor")) != NULL) {
+      int new_alloc=0;
+      char spec[MAXSTRING];
+
+      UtilStrCpyN(spec, sizeof(spec), c_ptr);
+      UtilTrimBlanks(spec);
+
+      if (QuickFindColorIndex(NULL, spec, &new_alloc, FALSE) == INVALID) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+               TOOL_NAME, "PNGExportTransparentColor", spec);
+         fprintf(stderr, "\n");
+         return;
+      }
+      UtilStrCpyN(pngExportTransparentColor, sizeof(pngExportTransparentColor), spec);
+   }
 }
 
 static
@@ -1229,6 +1626,10 @@ int ChangeObjColor(ObjPtr, ColorIndex)
          }
       } else if (ObjPtr->color != ColorIndex) {
          ObjPtr->color = ColorIndex;
+         if (mainDisplay != NULL) {
+            UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+                  colorMenuItems[ObjPtr->color]);
+         }
          if (ObjPtr->type == OBJ_XPM) {
             /* RecolorXPmObj(ObjPtr, ColorIndex); */
          }
@@ -1364,7 +1765,7 @@ void ChangeAllSelColor(ColorIndex, HighLight)
    if (changed) {
       int need_to_update_sel_bbox=FALSE;
 
-      if (colorLayers) {
+      if (!PRTGIF && colorLayers) {
          struct SelRec *next_sel;
 
          for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=next_sel) {
@@ -1387,6 +1788,9 @@ void ChangeAllSelColor(ColorIndex, HighLight)
                free(sel_ptr);
             }
          }
+         if (needToRedrawColorWindow) {
+            RedrawColorWindow();
+         }
       }
       SetFileModified(TRUE);
       RedrawAnArea(botObj, selLtX-GRID_ABS_SIZE(1),
@@ -1543,7 +1947,7 @@ void CleanUpColors()
    defaultColorIndex = 9;
    colorIndex = 0;
 
-   if (upend_bits) { }
+   if (upend_bits == NULL) { }
 }
 
 static
@@ -1785,7 +2189,6 @@ void ColorDummyEventHandler(input)
 
 typedef struct tagUpdatePixelRec {
    int picture_changed;
-   char **old_color_str;
 } UpdatePixelInfo;
 
 static
@@ -1795,25 +2198,27 @@ void UpdatePixelForStrSeg(pStrSeg, pUserData)
 {
    UpdatePixelInfo *pupi=((UpdatePixelInfo*)pUserData);
    int new_alloc=FALSE;
-   int index=QuickFindColorIndex(NULL, pupi->old_color_str[pStrSeg->color],
-         &new_alloc, TRUE);
+   int index=QuickFindColorIndex(NULL, pStrSeg->color_str, &new_alloc, TRUE);
 
    if (index != pStrSeg->color) pupi->picture_changed = TRUE;
    if (index == INVALID) {
       sprintf(gszMsgBox,
             TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-            pupi->old_color_str[pStrSeg->color], colorMenuItems[colorIndex]);
+            pStrSeg->color_str, colorMenuItems[colorIndex]);
       Msg(gszMsgBox);
       allocColorFailed = TRUE;
       pStrSeg->color = colorIndex;
    } else {
       pStrSeg->color = index;
    }
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(pStrSeg->color_str, sizeof(pStrSeg->color_str),
+            colorMenuItems[pStrSeg->color]);
+   }
 }
 
-int UpdatePixel(ObjPtr, OldColorStr)
+int UpdatePixel(ObjPtr)
    struct ObjRec *ObjPtr;
-   char **OldColorStr;
 {
    int c, i, r;
    int ncolors=0, new_alloc=FALSE, index=0, changed=FALSE, len=0;
@@ -1830,25 +2235,29 @@ int UpdatePixel(ObjPtr, OldColorStr)
    case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->last; obj_ptr!=NULL;
             obj_ptr=obj_ptr->prev) {
-         if (UpdatePixel(obj_ptr, OldColorStr)) {
+         if (UpdatePixel(obj_ptr)) {
             picture_changed = TRUE;
          }
       }
       break;
    case OBJ_XPM:
-      index = QuickFindColorIndex(NULL, OldColorStr[ObjPtr->color],
+      index = QuickFindColorIndex(NULL, ObjPtr->color_str,
             &new_alloc, TRUE);
       if (index != ObjPtr->color) picture_changed = TRUE;
       if (index == INVALID) {
          sprintf(gszMsgBox,
                TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-               OldColorStr[ObjPtr->color], colorMenuItems[colorIndex]);
+               ObjPtr->color_str, colorMenuItems[colorIndex]);
          Msg(gszMsgBox);
          allocColorFailed = TRUE;
          ObjPtr->color = colorIndex;
       } else {
          ObjPtr->color = index;
       }
+      if (mainDisplay != NULL) {
+         UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+               colorMenuItems[ObjPtr->color]);
+      }
       changed = FALSE;
       xpm_ptr = ObjPtr->detail.xpm;
       ncolors = xpm_ptr->ncolors;
@@ -1923,25 +2332,28 @@ int UpdatePixel(ObjPtr, OldColorStr)
       free(to_pixels);
       break;
    default:
-      index = QuickFindColorIndex(NULL, OldColorStr[ObjPtr->color],
+      index = QuickFindColorIndex(NULL, ObjPtr->color_str,
             &new_alloc, TRUE);
       if (index != ObjPtr->color) picture_changed = TRUE;
       if (index == INVALID) {
          sprintf(gszMsgBox,
                TgLoadCachedString(CSTID_CANNOT_ALLOC_COLOR_USE_ALT),
-               OldColorStr[ObjPtr->color], colorMenuItems[colorIndex]);
+               ObjPtr->color_str, colorMenuItems[colorIndex]);
          Msg(gszMsgBox);
          allocColorFailed = TRUE;
          ObjPtr->color = colorIndex;
       } else {
          ObjPtr->color = index;
       }
+      if (mainDisplay != NULL) {
+         UtilStrCpyN(ObjPtr->color_str, sizeof(ObjPtr->color_str),
+               colorMenuItems[ObjPtr->color]);
+      }
       if (ObjPtr->type == OBJ_TEXT) {
          UpdatePixelInfo upi;
 
          memset(&upi, 0, sizeof(UpdatePixelInfo));
          upi.picture_changed = FALSE;
-         upi.old_color_str = OldColorStr;
          DoFuncOnStrSegForMiniLines(&ObjPtr->detail.t->minilines,
                UpdatePixelForStrSeg, &upi);
          if (upi.picture_changed) {
@@ -1951,7 +2363,7 @@ int UpdatePixel(ObjPtr, OldColorStr)
       break;
    }
    for (attr_ptr=ObjPtr->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
-      if (UpdatePixel(attr_ptr->obj, OldColorStr)) {
+      if (UpdatePixel(attr_ptr->obj)) {
          picture_changed = TRUE;
       }
    }
@@ -1986,10 +2398,8 @@ void UpdateXPmObjects(ObjPtr)
 
 int FlushColormap()
 {
-   register int i;
-   int old_allocated_colors, changed=FALSE;
-   char **old_color_str;
-   struct ObjRec *obj_ptr;
+   int changed=FALSE;
+   struct ObjRec *obj_ptr=NULL;
 
 #ifdef DONT_FREE_COLORMAP
    Msg(TgLoadString(STID_COLORMAP_NOT_RESETED));
@@ -2012,15 +2422,6 @@ int FlushColormap()
             UpdateXPmObjects(obj_ptr);
          }
       }
-      old_color_str = (char**)malloc(maxColors*sizeof(char*));
-      if (old_color_str == NULL) FailAllocMessage();
-      old_allocated_colors = maxColors;
-      for (i=0; i < maxColors; i++) {
-         old_color_str[i] =
-               (char*)malloc((strlen(colorMenuItems[i])+1)*sizeof(char));
-         if (old_color_str[i] == NULL) FailAllocMessage();
-         strcpy(old_color_str[i], colorMenuItems[i]);
-      }
       initColorDontReload = TRUE;
       CleanUpColors();
       XFreeColormap(mainDisplay, mainColormap);
@@ -2073,7 +2474,7 @@ int FlushColormap()
       gnUpdatePixelObjCount = 0;
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
@@ -2081,14 +2482,12 @@ int FlushColormap()
       if (tgifObj->fattr != NULL) {
          for (attr_ptr=tgifObj->fattr; attr_ptr != NULL;
                attr_ptr=attr_ptr->next) {
-            if (UpdatePixel(attr_ptr->obj, old_color_str)) {
+            if (UpdatePixel(attr_ptr->obj)) {
                changed = TRUE;
             }
          }
       }
       RestoreStatusStrings();
-      for (i=0; i < old_allocated_colors; i++) free(old_color_str[i]);
-      free(old_color_str);
 
       DestroyPinnedMenu(MENU_COLOR);
       if (colorLayers) {
@@ -2130,6 +2529,11 @@ void AddColor()
    }
 }
 
+void ChooseColor()
+{
+   TgColorWheel_HandleCntrlSpace(mainDisplay, drawWindow);
+}
+
 void SetAltEditTextBgColor()
 {
    char spec[MAXSTRING+1];
