@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.17 2009/11/02 17:03:54 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.25 2011/05/16 16:21:56 william Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -76,6 +76,7 @@ int	colorIndex=0;
 int	*colorPixels=NULL;
 int	*xorColorPixels=NULL;
 int	*colorLayerOn=NULL;
+char	defaultColorStr[COLORSTRLEN];
 char	myFgColorStr[COLORSTRLEN];
 char	myBgColorStr[COLORSTRLEN];
 char	myRubberBandColorStr[COLORSTRLEN];
@@ -101,11 +102,14 @@ char	altEditTextHighlightColorStr[COLORSTRLEN];
 int	altEditTextHighlightIndex=INVALID;
 int	useAltEditTextBgColor=FALSE;
 
+char	pngExportTransparentColor[MAXSTRING];
+
 static int colorWindowFirstIndex=0;
 static int colorItemH=12;
 
 static int	canChangeAttrColor=FALSE;
 static int	defaultColorIndexInXDefaults=FALSE;
+static int	defaultColorInXDefaults=FALSE;
 static int	allocatedMaxColors=MAXCOLORS;
 
 static char *defaultColorMenuItems[MAXCOLORS]={
@@ -455,14 +459,180 @@ int DoSetDefaultBgColor()
    return TRUE;
 }
 
+static int std_palette8[] = { 0x00, 0xff };
+static int std_palette27[] = { 0x00, 0x80, 0xff };
+static int std_palette64[] = { 0x00, 0x55, 0xaa, 0xff };
+static int std_palette216[] = { 0x00, 0x33, 0x66, 0x99, 0xcc, 0xff };
+static char *openoffice_galaxy_palette[] = {
+      /* black */
+      "#000000",
+      "#151a1f",
+      "#29333c",
+      "#4b5a66",
+      "#8b97a0",
+      "#bec5c9",
+      "#d8dcdf",
+      "#edf1f5",
+      "#ffffff",
+      /* blue-gray */
+      "#123349",
+      "#35556b",
+      "#5382a1",
+      "#a3b8cb",
+      "#dae3ea",
+      /* more-blue-gray */
+      "#003e6d",
+      "#005a9b",
+      "#0877bc",
+      "#71c0eb",
+      "#a4cef0",
+      "#ddedfa",
+      /* blue */
+      "#063d80",
+      "#0646a2",
+      "#0852d4",
+      "#085dff",
+      "#0fa0ff",
+      "#b8f1ff",
+      /* green */
+      "#063e00",
+      "#096000",
+      "#128902",
+      "#1cbb04",
+      "#8dff1a",
+      "#cfff9e",
+      /* yellow */
+      "#b2bc00",
+      "#7f7800",
+      "#46580b",
+      "#314005",
+      "#634900",
+      "#c69200",
+      /* orange */
+      "#f26a00",
+      "#faa217",
+      "#ffc726",
+      "#ffdf24",
+      "#fff67b",
+      "#ffff9e",
+      /* red */
+      "#6d0000",
+      "#8e0000",
+      "#c70909",
+      "#ff0000",
+      "#fc5b5b",
+      "#ffa199",
+      /* purple */
+      "#2e008f",
+      "#8866cc",
+      "#c2b0e5",
+      NULL
+};
+static char *openoffice_google_palette[] = {
+      /* black */
+      "#000000",
+      "#262626",
+      "#404040",
+      "#575757",
+      "#747474",
+      "#909090",
+      "#a8a8a8",
+      "#c5c5c5",
+      "#e6e6e6",
+      "#ffffff",
+      /* saturated */
+      "#ff0000",
+      "#ff9900",
+      "#ffff00",
+      "#98fe00",
+      "#00ff00",
+      "#00ffff",
+      "#0035ff",
+      "#9900ff",
+      "#ff00ff",
+      "#ff0187",
+      /* red */
+      "#660000",
+      "#990000",
+      "#cc0000",
+      "#f03939",
+      "#ea9999",
+      "#f4cccc",
+      /* orange */
+      "#783f04",
+      "#b45f06",
+      "#e69138",
+      "#f6b26b",
+      "#f9cb9c",
+      "#fce5cd",
+      /* yellow */
+      "#7f6b00",
+      "#bfa000",
+      "#f1d932",
+      "#ffe766",
+      "#ffec99",
+      "#fff6cc",
+      /* light green */
+      "#2c4a00",
+      "#4c7f00",
+      "#5f9f00",
+      "#84be2d",
+      "#acdb65",
+      "#d1e7b1",
+      /* green */
+      "#1c4e13",
+      "#26761d",
+      "#5ba84f",
+      "#84c47d",
+      "#b0d7a8",
+      "#d6ead3",
+      /* cyan */
+      "#0c343d",
+      "#134f5c",
+      "#45818e",
+      "#76a5af",
+      "#a2c4c9",
+      "#d0e0e3",
+      /* blue */
+      "#073763",
+      "#0b5394",
+      "#3d85c6",
+      "#6fa8dc",
+      "#9fc5e8",
+      "#cfe2f3",
+      /* purple */
+      "#20124d",
+      "#351c75",
+      "#674ea7",
+      "#8e7cc3",
+      "#b4a7d6",
+      "#d9d2e9",
+      /* magenta */
+      "#4c114a",
+      "#741b68",
+      "#a64d9b",
+      "#c27bbd",
+      "#d5a6d4",
+      "#ead1ea",
+      /* redish */
+      "#5a002f",
+      "#94004e",
+      "#c10066",
+      "#e53b94",
+      "#e895c1",
+      "#f6d1e5",
+      NULL
+};
+
 void InitColor()
 {
-   int i, index, looking_for_default_color, tmp_max, color_from_pixmap=FALSE;
+   int i, index=0, looking_for_default_color=FALSE, tmp_max, color_from_pixmap=FALSE;
    XColor color;
    char buf[80], *c_ptr, fg_color[80], bg_color[80], brdr_color[80];
    long bg_gray=(long)0;
    int bg_allocated=FALSE, fg_allocated=FALSE, brdr_allocated=FALSE, ncolors=0;
-   char **color_str=NULL;
+   int std_palette=0, need_to_handle_additional_colors=FALSE;
+   char **color_str=NULL, *additional_colors_buf=NULL;
 
    InitLocalRGBTxt();
 
@@ -486,29 +656,79 @@ void InitColor()
          colorDump = FALSE;
       }
    }
+   defaultColorInXDefaults = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColor"))
+         != NULL) {
+      defaultColorInXDefaults = TRUE;
+      UtilStrCpyN(defaultColorStr, sizeof(defaultColorStr), c_ptr);
+   }
    defaultColorIndexInXDefaults = FALSE;
    defaultColorIndex = 9;
-   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColorIndex"))
-         != NULL) {
-      defaultColorIndexInXDefaults = TRUE;
-      defaultColorIndex = atoi(c_ptr);
+   if (!defaultColorInXDefaults) {
+      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DefaultColorIndex"))
+            != NULL) {
+         defaultColorIndexInXDefaults = TRUE;
+         defaultColorIndex = atoi(c_ptr);
+      }
    }
    color_from_pixmap = InitColorFromXPixmap(&ncolors, &color_str);
    if (color_from_pixmap && ncolors > 0 && color_str != NULL) {
       maxColors = ncolors;
+      need_to_handle_additional_colors = TRUE;
    } else {
-      maxColors = MAXCOLORS;
-      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "MaxColors")) != NULL) {
-         maxColors = atoi(c_ptr);
-         if (maxColors <= 0) {
-            fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
-                  TOOL_NAME, "MaxColors", c_ptr, MAXCOLORS);
-            fprintf(stderr, "\n");
-            maxColors = MAXCOLORS;
+      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseMobileWebSafePalette")) != NULL) {
+         maxColors = std_palette = 216;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette216")) != NULL) {
+         maxColors = std_palette = 216;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette64")) != NULL) {
+         maxColors = std_palette = 64;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette27")) != NULL) {
+         maxColors = std_palette = 27;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseStdPalette8")) != NULL) {
+         maxColors = std_palette = 8;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseOpenOfficeGalaxyPalette")) != NULL) {
+         maxColors = std_palette = 53;
+      } else if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+            "UseOpenOfficeGooglePalette")) != NULL) {
+         maxColors = std_palette = 80;
+      } else {
+         maxColors = MAXCOLORS;
+         if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "MaxColors")) != NULL) {
+            maxColors = atoi(c_ptr);
+            if (maxColors <= 0) {
+               fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+                     TOOL_NAME, "MaxColors", c_ptr, MAXCOLORS);
+               fprintf(stderr, "\n");
+               maxColors = MAXCOLORS;
+            }
          }
       }
+      if (std_palette > 0) {
+         need_to_handle_additional_colors = TRUE;
+      }
    }
+   if (need_to_handle_additional_colors) {
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"AdditionalColors")) !=
+            NULL) {
+         char *buf_copy=UtilStrDup(c_ptr), *psz_state=NULL;
 
+         if (buf_copy == NULL) FailAllocMessage();
+         UtilTrimBlanks(buf_copy);
+         additional_colors_buf = UtilStrDup(buf_copy);
+         if (additional_colors_buf == NULL) FailAllocMessage();
+         for (c_ptr=UtilStrTok(buf_copy, ",\t\n\r", &psz_state); c_ptr != NULL;
+               c_ptr=UtilStrTok(NULL, ",\t\n\r", &psz_state)) {
+            maxColors++;
+         }
+         UtilFree(buf_copy);
+      }
+   }
    if (colorDisplay) {
       if (((cmdLineForeground != NULL && cmdLineBackground == NULL) ||
             (cmdLineForeground == NULL && cmdLineBackground != NULL)) &&
@@ -597,24 +817,146 @@ void InitColor()
       free(color_str);
       color_str = NULL;
    } else {
-      for (i=0; i < maxColors; i++) {
-         sprintf(buf, "Color%1d", i);
-         if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, buf)) != NULL) {
-            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
-         } else if (i < MAXCOLORS) {
-            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN,
-                  defaultColorMenuItems[i]);
-         } else {
-            fprintf(stderr, TgLoadString(STID_CANNOT_GET_X_DEFAULT),
-                  TOOL_NAME, buf);
-            fprintf(stderr, "\n");
-            exit(-1);
+      int r=0, g=0, b=0;
+
+      switch (std_palette) {
+      case 0:
+         for (i=0; i < maxColors; i++) {
+            sprintf(buf, "Color%1d", i);
+            if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, buf)) != NULL) {
+               UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
+            } else if (i < MAXCOLORS) {
+               UtilStrCpyN(colorMenuItems[i], COLORSTRLEN,
+                     defaultColorMenuItems[i]);
+            } else {
+               fprintf(stderr, TgLoadString(STID_CANNOT_GET_X_DEFAULT),
+                     TOOL_NAME, buf);
+               fprintf(stderr, "\n");
+               exit(-1);
+            }
+            UtilTrimBlanks(colorMenuItems[i]);
+            colorLayerOn[i] = TRUE;
          }
-         UtilTrimBlanks(colorMenuItems[i]);
-         colorLayerOn[i] = TRUE;
+         break;
+      case 8:
+         i = 0;
+         for (r=0; r < 2; r++) {
+            for (g=0; g < 2; g++) {
+               for (b=0; b < 2; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette8[r], std_palette8[g], std_palette8[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 7;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 27:
+         i = 0;
+         for (r=0; r < 3; r++) {
+            for (g=0; g < 3; g++) {
+               for (b=0; b < 3; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette27[r], std_palette27[g], std_palette27[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 26;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 64:
+         i = 0;
+         for (r=0; r < 4; r++) {
+            for (g=0; g < 4; g++) {
+               for (b=0; b < 4; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette64[r], std_palette64[g], std_palette64[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 63;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 216:
+         i = 0;
+         for (r=0; r < 6; r++) {
+            for (g=0; g < 6; g++) {
+               for (b=0; b < 6; b++) {
+                  sprintf(colorMenuItems[i], "#%02x%02x%02x",
+                        std_palette216[r], std_palette216[g], std_palette216[b]);
+                  colorLayerOn[i] = TRUE;
+                  i++;
+               }
+            }
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 215;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 53: /* OpenOffice Galaxy Palette */
+         for (i=0; i < std_palette; i++) {
+            strcpy(colorMenuItems[i], openoffice_galaxy_palette[i]);
+            colorLayerOn[i] = TRUE;
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 8;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
+      case 80: /* OpenOffice Google Palette */
+         for (i=0; i < std_palette; i++) {
+            strcpy(colorMenuItems[i], openoffice_google_palette[i]);
+            colorLayerOn[i] = TRUE;
+         }
+         if (!defaultColorIndexInXDefaults) {
+            if (reverseVideo) {
+               defaultColorIndex = 9;
+            } else {
+               defaultColorIndex = 0;
+            }
+         }
+         break;
       }
    }
+   if (need_to_handle_additional_colors && additional_colors_buf != NULL) {
+      char *psz_state=NULL;
 
+      for (c_ptr=UtilStrTok(additional_colors_buf, ",\t\n\r", &psz_state); c_ptr != NULL;
+            c_ptr=UtilStrTok(NULL, ",\t\n\r", &psz_state)) {
+         UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
+         colorLayerOn[i] = TRUE;
+         i++;
+      }
+      UtilFree(additional_colors_buf);
+   }
    if (threeDLook) {
       static int initVryLtPixel=FALSE;
 
@@ -729,7 +1071,16 @@ void InitColor()
          exit (-1);
       }
       looking_for_default_color = FALSE;
-      if (defaultColorIndexInXDefaults) {
+      if (defaultColorInXDefaults) {
+         looking_for_default_color = TRUE;
+         for (i=0; i < maxColors; i++) {
+            if (UtilStrICmp(defaultColorStr, colorMenuItems[i]) == 0) {
+               defaultColorIndex = i;
+               looking_for_default_color = FALSE;
+               break;
+            }
+         }
+      } else if (defaultColorIndexInXDefaults) {
          if (defaultColorIndexInXDefaults && defaultColorIndex >= maxColors) {
             fprintf(stderr, TgLoadString(STID_DEFCOLORINDEX_GE_MAXCOLORS),
                   TOOL_NAME, "DefaultColorIndex", TOOL_NAME, "MaxColors",
@@ -775,6 +1126,35 @@ void InitColor()
                   ((long)color.blue);
          }
       }
+      if (defaultColorInXDefaults && looking_for_default_color) {
+         if (!ParseAndAllocColorByName(defaultColorStr, &color, NULL, NULL, NULL)) {
+            fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+                  TOOL_NAME, "DefaultColor", defaultColorStr);
+            fprintf(stderr, "\n");
+            defaultColorInXDefaults = FALSE;
+         }
+         for (i=0; i < maxColors; i++) {
+            if (color.pixel == colorPixels[i]) {
+               defaultColorIndex = i;
+               looking_for_default_color = FALSE;
+               break;
+            }
+         }
+         if (looking_for_default_color) {
+            int new_alloc=FALSE;
+
+            index = QuickFindColorIndex(NULL, defaultColorStr, &new_alloc, FALSE);
+            if (index == INVALID) {
+               fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+                     TOOL_NAME, "DefaultColor", defaultColorStr);
+               fprintf(stderr, "\n");
+               defaultColorInXDefaults = FALSE;
+            } else {
+               defaultColorIndex = index;
+               looking_for_default_color = FALSE;
+            }
+         }
+      }
       if (looking_for_default_color) {
          long val, val1;
 
@@ -949,6 +1329,23 @@ void InitColor()
          colorIndex = color_index;
       }
    }
+   UtilStrCpyN(pngExportTransparentColor, sizeof(pngExportTransparentColor),
+         myBgColorStr);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PNGExportTransparentColor")) != NULL) {
+      int new_alloc=0;
+      char spec[MAXSTRING];
+
+      UtilStrCpyN(spec, sizeof(spec), c_ptr);
+      UtilTrimBlanks(spec);
+
+      if (QuickFindColorIndex(NULL, spec, &new_alloc, FALSE) == INVALID) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+               TOOL_NAME, "PNGExportTransparentColor", spec);
+         fprintf(stderr, "\n");
+         return;
+      }
+      UtilStrCpyN(pngExportTransparentColor, sizeof(pngExportTransparentColor), spec);
+   }
 }
 
 static
