@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dialog.c,v 1.8 2009/08/13 01:14:20 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dialog.c,v 1.10 2011/05/16 16:21:57 william Exp $
  */
 
 #define _INCLUDE_FROM_DIALOG_C_
@@ -705,6 +705,8 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
 #endif /* NOTR4MODE */
    RegisterWM_DELETE_WINDOW(MBInfoPtr->main_win);
    XStoreName(mainDisplay, MBInfoPtr->main_win, Title);
+   /* so that MessageBox() and Dialog() window stays on top of tgif's window */
+   XSetTransientForHint(mainDisplay, MBInfoPtr->main_win, mainWindow);
 
 #ifdef MAPBEFORESELECT
    XMapWindow(mainDisplay, MBInfoPtr->main_win);
@@ -1170,6 +1172,9 @@ int MsgBox(Message, Title, IconAndBtns)
       }
       if (input.type == Expose) {
          ExposeEventHandler(&input, FALSE);
+      } else if (input.type == ConfigureNotify &&
+            input.xany.window == mainWindow) {
+         Reconfigure(FALSE);
       } else if (input.type == VisibilityNotify &&
             input.xany.window==mainWindow &&
             input.xvisibility.state==VisibilityUnobscured) {
@@ -1284,6 +1289,9 @@ int DoDialog(Message, ReturnStr)
       }
       if (input.type == Expose) {
          ExposeEventHandler(&input, FALSE);
+      } else if (input.type == ConfigureNotify &&
+            input.xany.window == mainWindow) {
+         Reconfigure(FALSE);
       } else if (input.type == VisibilityNotify &&
             input.xany.window==mainWindow &&
             input.xvisibility.state==VisibilityUnobscured) {
