@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.13 2011/05/16 16:21:58 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.14 2011/06/14 02:32:18 william Exp $
  */
 
 #define _INCLUDE_FROM_INMETHOD_C_
@@ -157,7 +157,7 @@ int XcinFocusIn(dpy, win)
    Display *dpy;
    Window win;
 {
-   send_FocusIn(dpy, win);
+   Tg_send_FocusIn(dpy, win);
    return TRUE;
 }
 
@@ -166,7 +166,7 @@ int XcinFocusOut(dpy, win)
    Display *dpy;
    Window win;
 {
-   send_FocusOut(dpy, win);
+   Tg_send_FocusOut(dpy, win);
    return TRUE;
 }
 
@@ -177,7 +177,7 @@ int XcinSendKey(dpy, win, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   return send_key(dpy, win, key_ev, buf);
+   return Tg_send_key(dpy, win, key_ev, buf);
 }
 
 static
@@ -198,7 +198,7 @@ int ChinputInit(dpy, win, arg)
    Window win;
    char *arg;
 {
-   return (HZclientInit(dpy)==0);
+   return (Tg_HZclientInit(dpy)==0);
 }
 
 static
@@ -224,7 +224,7 @@ int ChinputSendKey(dpy, win, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   HZsendKey(dpy, win, key_ev);
+   Tg_HZsendKey(dpy, win, key_ev);
    *buf = '\0';
    return TRUE;
 }
@@ -245,23 +245,7 @@ int ChinputHandleCM(dpy, win, cm_ev, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   char s1[80];
-   int num=0, rc=0;
-
-   rc = HZprocInput(cm_ev, &num, &key_ev->keycode, &key_ev->state, s1);
-
-   if (rc == HZSERVER_OUTPUT_BAD) {
-      return FALSE;
-   } else if (rc == HZSERVER_OUTPUT_NORMAL_KEY) {
-      buf[0] = key_ev->keycode;
-      buf[1] = '\0';
-   } else if (rc == HZSERVER_OUTPUT_FUNCTION_KEY) {
-      buf[0] = '\0';
-   } else if (rc == HZSERVER_OUTPUT_HANZI) {
-      strncpy(buf, s1, num);
-      buf[num] = '\0';
-   }
-   return TRUE;
+   return Tg_HZhandleCM(cm_ev, key_ev, buf);
 }
 #endif /* ~_NO_CHINPUT */
 
