@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.14 2011/06/14 02:32:18 william Exp $
  */
 
 #define _INCLUDE_FROM_INMETHOD_C_
@@ -46,6 +46,22 @@
 #include "strtbl.e"
 #include "tidget.e"
 
+#ifndef _NO_DL_SUPPORT
+#define CLOSE_DL(handle) dlclose(handle)
+#define OPEN_DL(path,flag) dlopen((path),(flag))
+#define GET_DL_SYM(handle,symbol) dlsym((handle),(symbol))
+#ifndef RTLD_NOW
+#define OPEN_DL_MODE 1
+#else /* RTLD_NOW */
+#define OPEN_DL_MODE (RTLD_NOW|RTLD_GLOBAL)
+#endif /* ~RTLD_NOW */
+#else /* _NO_DL_SUPPORT */
+#define CLOSE_DL(handle)
+#define OPEN_DL(path,flag) NULL
+#define GET_DL_SYM(handle,symbol) NULL
+#define OPEN_DL_MODE 0
+#endif /* ~_NO_DL_SUPPORT */
+
 int gnInputMethod=TGIM_NONE; /* one of TGIM_* */
 int gnOverTheSpot=FALSE;
 
@@ -128,7 +144,7 @@ int XcinFocusIn(dpy, win)
    Display *dpy;
    Window win;
 {
-   send_FocusIn(dpy, win);
+   Tg_send_FocusIn(dpy, win);
    return TRUE;
 }
 
@@ -137,7 +153,7 @@ int XcinFocusOut(dpy, win)
    Display *dpy;
    Window win;
 {
-   send_FocusOut(dpy, win);
+   Tg_send_FocusOut(dpy, win);
    return TRUE;
 }
 
@@ -148,7 +164,7 @@ int XcinSendKey(dpy, win, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   return send_key(dpy, win, key_ev, buf);
+   return Tg_send_key(dpy, win, key_ev, buf);
 }
 
 static
@@ -169,7 +185,7 @@ int ChinputInit(dpy, win, arg)
    Window win;
    char *arg;
 {
-   return (HZclientInit(dpy)==0);
+   return (Tg_HZclientInit(dpy)==0);
 }
 
 static
@@ -195,7 +211,7 @@ int ChinputSendKey(dpy, win, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   HZsendKey(dpy, win, key_ev);
+   Tg_HZsendKey(dpy, win, key_ev);
    *buf = '\0';
    return TRUE;
 }
@@ -216,23 +232,7 @@ int ChinputHandleCM(dpy, win, cm_ev, key_ev, buf)
    XKeyEvent *key_ev;
    char *buf;
 {
-   char s1[80];
-   int num=0, rc=0;
-
-   rc = HZprocInput(cm_ev, &num, &key_ev->keycode, &key_ev->state, s1);
-
-   if (rc == HZSERVER_OUTPUT_BAD) {
-      return FALSE;
-   } else if (rc == HZSERVER_OUTPUT_NORMAL_KEY) {
-      buf[0] = key_ev->keycode;
-      buf[1] = '\0';
-   } else if (rc == HZSERVER_OUTPUT_FUNCTION_KEY) {
-      buf[0] = '\0';
-   } else if (rc == HZSERVER_OUTPUT_HANZI) {
-      strncpy(buf, s1, num);
-      buf[num] = '\0';
-   }
-   return TRUE;
+   return Tg_HZhandleCM(cm_ev, key_ev, buf);
 }
 #endif /* ~_NO_CHINPUT */
 
@@ -623,7 +623,7 @@ static struct tagIMInfo gstIMInfo[] = {
    },
 #endif /* ~_NO_XIM */
 #ifndef _NO_TGTWB5
-   { "tgtwb5", TGIM_TGTWB5, TGIM_DBIM,
+   { "tgtwb5/tgchgb", TGIM_TGTWB5, TGIM_DBIM,
      Tgtwb5CleanUp, Tgtwb5Init, NULL, NULL,
      Tgtwb5SendKey, NULL, Tgtwb5ExpectCM, NULL,
      Tgtwb5ActiveOnCntrlSpace, Tgtwb5HandleCntrlSpace,
@@ -637,8 +637,60 @@ static struct tagIMInfo gstIMInfo[] = {
    }
 };
 
+static void *gpLibIdnHandle=NULL;
+static Tgim_stringprep_convert_Func *gpfn_stringprep_convert=NULL;
+
+static
+void CleanUpTgim_stringprep_convert()
+{
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert = NULL;
+      CLOSE_DL(gpLibIdnHandle);
+      gpLibIdnHandle = NULL;
+   }
+}
+
+static
+void InitTgim_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   gpLibIdnHandle = OPEN_DL("libidn.so", OPEN_DL_MODE);
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert =
+            (Tgim_stringprep_convert_Func*)GET_DL_SYM(gpLibIdnHandle,
+            "stringprep_convert");
+      if (gpfn_stringprep_convert == NULL) {
+         CLOSE_DL(gpLibIdnHandle);
+         gpLibIdnHandle = NULL;
+      }
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+}
+
+int Tgim_has_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+      return TRUE;
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return FALSE;
+}
+
+char *Tgim_stringprep_convert(const char *str, const char *to_codeset,
+      const char *from_codeset)
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+       return (gpfn_stringprep_convert)(str, to_codeset, from_codeset);
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return NULL;
+}
+
 void CleanUpInputMethods()
 {
+   CleanUpTgim_stringprep_convert();
    if (gnInputMethodIndex != INVALID &&
          gstIMInfo[gnInputMethodIndex].pCleanUpFunc != NULL) {
       (gstIMInfo[gnInputMethodIndex].pCleanUpFunc)(mainDisplay, mainWindow);
@@ -666,6 +718,14 @@ int InitInputMethods()
          psz = XGetDefault(mainDisplay, TOOL_NAME, "SingleByteInputMethod");
          single_double = TGIM_SBIM;
       } else {
+         if (XGetDefault(mainDisplay, TOOL_NAME, "SingleByteInputMethod") !=
+               NULL) {
+            sprintf(gszMsgBox, TgLoadString(STID_TWO_XDEF_ONE_IGNORED),
+                  TOOL_NAME, "DoubleByteInputMethod", TOOL_NAME,
+                  "SingleByteInputMethod", TOOL_NAME, "SingleByteInputMethod");
+            Msg(gszMsgBox);
+            fprintf(stderr, "%s\n", gszMsgBox);
+         }
          single_double = TGIM_DBIM;
       }
    }
@@ -678,25 +738,70 @@ int InitInputMethods()
 
       if (psz_comma != NULL) *psz_comma = '\0';
       for ( ; pIMInfo->pszName != NULL; pIMInfo++, i++) {
-         if (UtilStrICmp(psz, pIMInfo->pszName) == 0 &&
-               (single_double & pIMInfo->nSingleDouble) == single_double) {
-            gnInputMethod = pIMInfo->nInputMethod;
-            gnInputMethodIndex = i;
-            gpIMInfo = pIMInfo;
-            if (gstIMInfo[gnInputMethodIndex].pInitFunc != NULL) {
-               if (psz_comma == NULL) {
-                  if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
-                        mainWindow, NULL))) {
-                     /* may not be a problem */
-                  }
-               } else {
-                  if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
-                        mainWindow, &psz_comma[1]))) {
-                     /* may not be a problem */
+         char *psz_slash=strchr(pIMInfo->pszName,'/');
+
+         if (psz_slash == NULL) {
+            if (UtilStrICmp(psz, pIMInfo->pszName) == 0 &&
+                  (single_double & pIMInfo->nSingleDouble) == single_double) {
+               gnInputMethod = pIMInfo->nInputMethod;
+               gnInputMethodIndex = i;
+               gpIMInfo = pIMInfo;
+               if (gstIMInfo[gnInputMethodIndex].pInitFunc != NULL) {
+                  if (psz_comma == NULL) {
+                     if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                           mainWindow, NULL))) {
+                        /* may not be a problem */
+                     }
+                  } else {
+                     if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                           mainWindow, &psz_comma[1]))) {
+                        /* may not be a problem */
+                     }
                   }
                }
+               break;
+            }
+         } else {
+            char *psz_dup=UtilStrDup(pIMInfo->pszName), *psz_cur=NULL;
+            int done=FALSE;
+
+            if (psz_dup == NULL) FailAllocMessage();
+            psz_cur = psz_dup;
+            psz_slash = strchr(psz_cur, '/');
+            while (!done) {
+               if (psz_slash != NULL) *psz_slash = '\0';
+               if (UtilStrICmp(psz, psz_cur) == 0 &&
+                     (single_double & pIMInfo->nSingleDouble) == single_double) {
+                  gnInputMethod = pIMInfo->nInputMethod;
+                  gnInputMethodIndex = i;
+                  gpIMInfo = pIMInfo;
+                  if (gstIMInfo[gnInputMethodIndex].pInitFunc != NULL) {
+                     if (psz_comma == NULL) {
+                        if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                              mainWindow, NULL))) {
+                           /* may not be a problem */
+                        }
+                     } else {
+                        if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                              mainWindow, &psz_comma[1]))) {
+                           /* may not be a problem */
+                        }
+                     }
+                  }
+                  done = TRUE;
+               }
+               if (psz_slash != NULL) {
+                  *psz_slash++ = '/';
+                  psz_cur = psz_slash;
+                  psz_slash = strchr(psz_cur, '/');
+               } else {
+                  break;
+               }
+            }
+            UtilFree(psz_dup);
+            if (done) {
+               break;
             }
-            break;
          }
       }
       if (psz_comma != NULL) *psz_comma++ = ',';
@@ -710,6 +815,10 @@ int InitInputMethods()
          fprintf(stderr, "\n");
       }
    }
+   if (gnSingleOrDoubleByteInputMethod == TGIM_SBIM &&
+         gnInputMethod != INVALID) {
+      InitTgim_stringprep_convert();
+   }
    return TRUE;
 }
 
