@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainmenu.c,v 1.4 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainmenu.c,v 1.8 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINMENU_C_
@@ -971,9 +971,11 @@ void UpdatePinnedMenu(menu_index)
             case MENU_PAGE: RefreshPageMenu(menu); break;
             case MENU_PAGELAYOUT: RefreshPageLayoutMenu(menu); break;
             case MENU_HELP: RefreshHelpMenu(menu); break;
-#ifdef _TANGRAM_II
-            case MENU_TANGRAM2: RefreshTangram2Menu(menu); break;
-#endif /* _TANGRAM_II */
+            case MENU_TANGRAM2:
+               if (cmdLineTgrm2) {
+                  RefreshTangram2Menu(menu);
+               }
+               break;
 
             case MENU_MODE: RefreshModeMenu(menu); break;
 
