@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mark.c,v 4.17 2002/01/05 02:48:02 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mark.c,v 1.25 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_MARK_C_
@@ -42,12 +42,11 @@
 #include "rect.e"
 #include "setup.e"
 #include "select.e"
+#include "spline.e"
+#include "stretch.e"
 #include "strtbl.e"
 #include "util.e"
 
-#define FORWARD 0
-#define REVERSE 1
-
 int somethingHighLighted=FALSE;
 
 static int tickMarkSize=8;
@@ -67,12 +66,66 @@ int InitMark()
       if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"TickMarkSize")) != NULL) {
          SetTickMarkSizeValue(c_ptr);
       }
+      autoRotatePivot = FALSE;
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"AutoRotatePivot")) !=
+            NULL && UtilStrICmp(c_ptr, "true") == 0) {
+         autoRotatePivot = TRUE;
+         choicePixmap[ROTATEMODE] = rotateModePixmap[!autoRotatePivot];
+      }
    }
    return TRUE;
 }
 
 /* --------------------- HighLight Functions --------------------- */
 
+static
+void MarkStructuredSpline(obj_ptr, ssn, ssvs, ssmooth, curved, locked)
+   struct ObjRec *obj_ptr;
+   int ssn, curved, locked;
+   char *ssmooth;
+   IntPoint *ssvs;
+{
+   int i=0;
+
+#ifdef _TGIF_DBG /* debug, do not translate */
+   TgAssert(curved==LT_STRUCT_SPLINE,
+         "curved != LT_STRUCT_SPLINE in MarkStructuredSpline()", NULL);
+#endif /* _TGIF_DBG */
+
+   if (obj_ptr->ctm == NULL) {
+      if (ssmooth != NULL) {
+         for (i = 0; i < ssn; i++) {
+            if (ssmooth[i]) {
+               MARKHO(drawWindow, locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(ssvs[i].x), OFFSET_Y(ssvs[i].y));
+            } else {
+               MARKHR(drawWindow, locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(ssvs[i].x), OFFSET_Y(ssvs[i].y));
+            }
+         }
+      } else {
+         for (i = 0; i < ssn; i++) {
+            MARKHR(drawWindow, locked ? revGrayGC : revDefaultGC,
+                  OFFSET_X(ssvs[i].x), OFFSET_Y(ssvs[i].y));
+         }
+      }
+   } else {
+      for (i=0; i < ssn; i++) {
+         int x, y;
+
+         TransformPointThroughCTM(ssvs[i].x-obj_ptr->x, ssvs[i].y-obj_ptr->y,
+               obj_ptr->ctm, &x, &y);
+         if (ssmooth != NULL && ssmooth[i]) {
+            MARKHO(drawWindow, locked ? revGrayGC : revDefaultGC,
+                  OFFSET_X(x+obj_ptr->x), OFFSET_Y(y+obj_ptr->y));
+         } else {
+            MARKHR(drawWindow, locked ? revGrayGC : revDefaultGC,
+                  OFFSET_X(x+obj_ptr->x), OFFSET_Y(y+obj_ptr->y));
+         }
+      }
+   }
+}
+
 static
 void MarkPoly(ObjPtr, NumPts, V, Smooth, Curved, Locked)
    struct ObjRec *ObjPtr;
@@ -86,11 +139,21 @@ void MarkPoly(ObjPtr, NumPts, V, Smooth, Curved, Locked)
       if (Curved != LT_INTSPLINE && Smooth != NULL) {
          for (i = 0; i < NumPts; i++) {
             if (Smooth[i]) {
-               MARKO(drawWindow, Locked ? revGrayGC : revDefaultGC,
-                     OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               if (Curved == LT_STRUCT_SPLINE) {
+                  MARKHO(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                        OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               } else {
+                  MARKO(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                        OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               }
             } else {
-               MARK(drawWindow, Locked ? revGrayGC : revDefaultGC,
-                     OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               if (Curved == LT_STRUCT_SPLINE) {
+                  MARKHR(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                        OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               } else {
+                  MARK(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                        OFFSET_X(V[i].x), OFFSET_Y(V[i].y));
+               }
             }
          }
       } else {
@@ -106,11 +169,21 @@ void MarkPoly(ObjPtr, NumPts, V, Smooth, Curved, Locked)
          TransformPointThroughCTM(V[i].x-ObjPtr->x, V[i].y-ObjPtr->y,
                ObjPtr->ctm, &x, &y);
          if (Curved != LT_INTSPLINE && Smooth != NULL && Smooth[i]) {
-            MARKO(drawWindow, Locked ? revGrayGC : revDefaultGC,
-                  OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            if (Curved == LT_STRUCT_SPLINE) {
+               MARKHO(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            } else {
+               MARKO(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            }
          } else {
-            MARK(drawWindow, Locked ? revGrayGC : revDefaultGC,
-                  OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            if (Curved == LT_STRUCT_SPLINE) {
+               MARKHR(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            } else {
+               MARK(drawWindow, Locked ? revGrayGC : revDefaultGC,
+                     OFFSET_X(x+ObjPtr->x), OFFSET_Y(y+ObjPtr->y));
+            }
          }
       }
    }
@@ -195,14 +268,26 @@ void HighLightAnObj(ObjPtr)
    if (execCurDepth > 0) return;
    switch (ObjPtr->type) {
    case OBJ_POLY:
-      MarkPoly(ObjPtr, ObjPtr->detail.p->n, ObjPtr->detail.p->vlist,
-            ObjPtr->detail.p->smooth, ObjPtr->detail.p->curved,
-            ObjPtr->locked);
+      if (ObjPtr->detail.p->curved == LT_STRUCT_SPLINE) {
+         MarkStructuredSpline(ObjPtr, ObjPtr->detail.p->ssn,
+               ObjPtr->detail.p->ssvlist, ObjPtr->detail.p->ssmooth,
+               ObjPtr->detail.p->curved, ObjPtr->locked);
+      } else {
+         MarkPoly(ObjPtr, ObjPtr->detail.p->n, ObjPtr->detail.p->vlist,
+               ObjPtr->detail.p->smooth, ObjPtr->detail.p->curved,
+               ObjPtr->locked);
+      }
       break;
    case OBJ_POLYGON:
-      MarkPoly(ObjPtr, ObjPtr->detail.g->n-1, ObjPtr->detail.g->vlist,
-            ObjPtr->detail.g->smooth, ObjPtr->detail.g->curved,
-            ObjPtr->locked);
+      if (ObjPtr->detail.g->curved == LT_STRUCT_SPLINE) {
+         MarkStructuredSpline(ObjPtr, ObjPtr->detail.g->ssn-1,
+               ObjPtr->detail.g->ssvlist, ObjPtr->detail.g->ssmooth,
+               ObjPtr->detail.g->curved, ObjPtr->locked);
+      } else {
+         MarkPoly(ObjPtr, ObjPtr->detail.g->n-1, ObjPtr->detail.g->vlist,
+               ObjPtr->detail.g->smooth, ObjPtr->detail.g->curved,
+               ObjPtr->locked);
+      }
       break;
 
    case OBJ_BOX:
@@ -230,7 +315,8 @@ void HighLightVertices(Dir)
    register int i;
    struct SelRec *sel_ptr=NULL;
 
-   if (execCurDepth > 0) return;
+   if (mainDisplay == NULL || execCurDepth > 0) return;
+
    switch (Dir) {
    case FORWARD: vsel_ptr=botVSel; sel_ptr=botSel; break;
    case REVERSE: vsel_ptr=topVSel; sel_ptr=topSel; break;
@@ -240,14 +326,26 @@ void HighLightVertices(Dir)
 
       switch (obj_ptr->type) {
       case OBJ_POLY:
-         MarkPoly(obj_ptr, obj_ptr->detail.p->n, obj_ptr->detail.p->vlist,
-               obj_ptr->detail.p->smooth, obj_ptr->detail.p->curved,
-               obj_ptr->locked);
+         if (obj_ptr->detail.p->curved == LT_STRUCT_SPLINE) {
+            MarkStructuredSpline(obj_ptr, obj_ptr->detail.p->ssn,
+                  obj_ptr->detail.p->ssvlist, obj_ptr->detail.p->ssmooth,
+                  obj_ptr->detail.p->curved, obj_ptr->locked);
+         } else {
+            MarkPoly(obj_ptr, obj_ptr->detail.p->n, obj_ptr->detail.p->vlist,
+                  obj_ptr->detail.p->smooth, obj_ptr->detail.p->curved,
+                  obj_ptr->locked);
+         }
          break;
       case OBJ_POLYGON:
-         MarkPoly(obj_ptr, obj_ptr->detail.g->n-1, obj_ptr->detail.g->vlist,
-               obj_ptr->detail.g->smooth, obj_ptr->detail.g->curved,
-               obj_ptr->locked);
+         if (obj_ptr->detail.g->curved == LT_STRUCT_SPLINE) {
+            MarkStructuredSpline(obj_ptr, obj_ptr->detail.g->ssn-1,
+                  obj_ptr->detail.g->ssvlist, obj_ptr->detail.g->ssmooth,
+                  obj_ptr->detail.g->curved, obj_ptr->locked);
+         } else {
+            MarkPoly(obj_ptr, obj_ptr->detail.g->n-1, obj_ptr->detail.g->vlist,
+                  obj_ptr->detail.g->smooth, obj_ptr->detail.g->curved,
+                  obj_ptr->locked);
+         }
          break;
       }
       switch (Dir) {
@@ -261,12 +359,20 @@ void HighLightVertices(Dir)
 
       switch (vsel_ptr->obj->type) {
       case OBJ_POLY:
-         smooth = vsel_ptr->obj->detail.p->smooth;
          curved = vsel_ptr->obj->detail.p->curved;
+         if (curved == LT_STRUCT_SPLINE) {
+            smooth = vsel_ptr->obj->detail.p->ssmooth;
+         } else {
+            smooth = vsel_ptr->obj->detail.p->smooth;
+         }
          break;
       case OBJ_POLYGON:
-         smooth = vsel_ptr->obj->detail.g->smooth;
          curved = vsel_ptr->obj->detail.g->curved;
+         if (curved == LT_STRUCT_SPLINE) {
+            smooth = vsel_ptr->obj->detail.g->ssmooth;
+         } else {
+            smooth = vsel_ptr->obj->detail.g->smooth;
+         }
          break;
       }
       for (i = 0; i < vsel_ptr->n; i++) {
@@ -274,11 +380,21 @@ void HighLightVertices(Dir)
                vsel_ptr->obj->detail.g->n-1==vsel_ptr->v_index[i])) {
             if (curved != LT_INTSPLINE && curved != (-1) && smooth != NULL) {
                if (smooth[vsel_ptr->v_index[i]]) {
-                  MARKO(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
-                        OFFSET_Y(vsel_ptr->y[i]));
+                  if (curved == LT_STRUCT_SPLINE) {
+                     MARKHO(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
+                           OFFSET_Y(vsel_ptr->y[i]));
+                  } else {
+                     MARKO(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
+                           OFFSET_Y(vsel_ptr->y[i]));
+                  }
                } else {
-                  MARK(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
-                        OFFSET_Y(vsel_ptr->y[i]));
+                  if (curved == LT_STRUCT_SPLINE) {
+                     MARKHR(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
+                           OFFSET_Y(vsel_ptr->y[i]));
+                  } else {
+                     MARK(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
+                           OFFSET_Y(vsel_ptr->y[i]));
+                  }
                }
             } else {
                MARK(drawWindow, revDefaultGC, OFFSET_X(vsel_ptr->x[i]),
@@ -288,10 +404,9 @@ void HighLightVertices(Dir)
                   OFFSET_Y(vsel_ptr->y[i]));
          }
       }
-      switch (Dir)
-      {
-         case FORWARD: vsel_ptr = vsel_ptr->prev; break;
-         case REVERSE: vsel_ptr = vsel_ptr->next; break;
+      switch (Dir) {
+      case FORWARD: vsel_ptr = vsel_ptr->prev; break;
+      case REVERSE: vsel_ptr = vsel_ptr->next; break;
       }
    }
 }
@@ -303,7 +418,8 @@ void HighLight(Dir)
    register struct SelRec *sel_ptr=NULL;
    register struct ObjRec *obj_ptr;
 
-   if (execCurDepth > 0) return;
+   if (mainDisplay == NULL || execCurDepth > 0) return;
+
    switch (Dir) {
    case FORWARD: sel_ptr = botSel; break;
    case REVERSE: sel_ptr = topSel; break;
@@ -312,14 +428,26 @@ void HighLight(Dir)
       obj_ptr = sel_ptr->obj;
       switch (obj_ptr->type) {
       case OBJ_POLY:
-         MarkPoly(obj_ptr, obj_ptr->detail.p->n, obj_ptr->detail.p->vlist,
-               obj_ptr->detail.p->smooth, obj_ptr->detail.p->curved,
-               obj_ptr->locked);
+         if (obj_ptr->detail.p->curved == LT_STRUCT_SPLINE) {
+            MarkStructuredSpline(obj_ptr, obj_ptr->detail.p->ssn,
+                  obj_ptr->detail.p->ssvlist, obj_ptr->detail.p->ssmooth,
+                  obj_ptr->detail.p->curved, obj_ptr->locked);
+         } else {
+            MarkPoly(obj_ptr, obj_ptr->detail.p->n, obj_ptr->detail.p->vlist,
+                  obj_ptr->detail.p->smooth, obj_ptr->detail.p->curved,
+                  obj_ptr->locked);
+         }
          break;
       case OBJ_POLYGON:
-         MarkPoly(obj_ptr, obj_ptr->detail.g->n-1, obj_ptr->detail.g->vlist,
-               obj_ptr->detail.g->smooth, obj_ptr->detail.g->curved,
-               obj_ptr->locked);
+         if (obj_ptr->detail.g->curved == LT_STRUCT_SPLINE) {
+            MarkStructuredSpline(obj_ptr, obj_ptr->detail.g->ssn-1,
+                  obj_ptr->detail.g->ssvlist, obj_ptr->detail.g->ssmooth,
+                  obj_ptr->detail.g->curved, obj_ptr->locked);
+         } else {
+            MarkPoly(obj_ptr, obj_ptr->detail.g->n-1, obj_ptr->detail.g->vlist,
+                  obj_ptr->detail.g->smooth, obj_ptr->detail.g->curved,
+                  obj_ptr->locked);
+         }
          break;
 
       case OBJ_BOX:
@@ -349,6 +477,9 @@ void HighLightForward()
       HighLightVertices(FORWARD);
    } else {
       HighLight(FORWARD);
+      if (!autoRotatePivot && curChoice == ROTATEMODE) {
+         HighLightRotatePivot(FORWARD);
+      }
    }
    somethingHighLighted = TRUE;
 }
@@ -358,6 +489,9 @@ void HighLightReverse()
    if (curChoice == VERTEXMODE) {
       HighLightVertices(REVERSE);
    } else {
+      if (!autoRotatePivot && curChoice == ROTATEMODE) {
+         HighLightRotatePivot(REVERSE);
+      }
       HighLight(REVERSE);
    }
    somethingHighLighted = FALSE;
@@ -637,59 +771,71 @@ IntPoint *GetSplinePolyTickMarkVs(pn_return, n, vs, smooth)
    while (i < n) {
       if (smooth[i]) {
          int j=0, last_hinge_index=i, num_vs=0;
-         DoublePoint *mid_pt_of_mid_segs=NULL, dvs[3];
+         DoublePoint dvs[3];
 
          for (last_hinge_index=i+1; smooth[last_hinge_index];
                last_hinge_index++) {
          }
          num_vs = last_hinge_index-first_hinge_index+1;
-         mid_pt_of_mid_segs =
-               (DoublePoint*)malloc((num_vs-3)*sizeof(DoublePoint));
-         if (mid_pt_of_mid_segs == NULL) FailAllocMessage();
-         memset(mid_pt_of_mid_segs, 0, (num_vs-3)*sizeof(DoublePoint));
+         if (num_vs > 3) {
+            DoublePoint *mid_pt_of_mid_segs=NULL;
 
-         for (j=first_hinge_index+1; j < last_hinge_index-1; j++) {
-            /*
-             * These are the midpoints of imaginary line segments
-             * the spline actually runs through.
-             */
-            mid_pt_of_mid_segs[j-first_hinge_index-1].x =
-                  ((double)(vs[j].x + vs[j+1].x)) / ((double)2);
-            mid_pt_of_mid_segs[j-first_hinge_index-1].y =
-                  ((double)(vs[j].y + vs[j+1].y)) / ((double)2);
-         }
-         dvs[0].x = (double)(vs[first_hinge_index].x);
-         dvs[0].y = (double)(vs[first_hinge_index].y);
-         dvs[1].x = (double)(vs[first_hinge_index+1].x);
-         dvs[1].y = (double)(vs[first_hinge_index+1].y);
-         dvs[2].x = mid_pt_of_mid_segs[0].x;
-         dvs[2].y = mid_pt_of_mid_segs[0].y;
-         SetRealVertex(vs_return, write_index++, dvs);
-         vs_return[write_index].x = round(dvs[2].x);
-         vs_return[write_index].y = round(dvs[2].y);
-         write_index++;
-
-         for (j=first_hinge_index+2; j < last_hinge_index-1; j++) {
-            dvs[0].x = mid_pt_of_mid_segs[j-first_hinge_index-2].x;
-            dvs[0].y = mid_pt_of_mid_segs[j-first_hinge_index-2].y;
-            dvs[1].x = (double)(vs[j].x);
-            dvs[1].y = (double)(vs[j].y);
-            dvs[2].x = mid_pt_of_mid_segs[j-first_hinge_index-1].x;
-            dvs[2].y = mid_pt_of_mid_segs[j-first_hinge_index-1].y;
+            mid_pt_of_mid_segs =
+                  (DoublePoint*)malloc((num_vs-3)*sizeof(DoublePoint));
+            if (mid_pt_of_mid_segs == NULL) FailAllocMessage();
+            memset(mid_pt_of_mid_segs, 0, (num_vs-3)*sizeof(DoublePoint));
+   
+            for (j=first_hinge_index+1; j < last_hinge_index-1; j++) {
+               /*
+                * These are the midpoints of imaginary line segments
+                * the spline actually runs through.
+                */
+               mid_pt_of_mid_segs[j-first_hinge_index-1].x =
+                     ((double)(vs[j].x + vs[j+1].x)) / ((double)2);
+               mid_pt_of_mid_segs[j-first_hinge_index-1].y =
+                     ((double)(vs[j].y + vs[j+1].y)) / ((double)2);
+            }
+            dvs[0].x = (double)(vs[first_hinge_index].x);
+            dvs[0].y = (double)(vs[first_hinge_index].y);
+            dvs[1].x = (double)(vs[first_hinge_index+1].x);
+            dvs[1].y = (double)(vs[first_hinge_index+1].y);
+            dvs[2].x = mid_pt_of_mid_segs[0].x;
+            dvs[2].y = mid_pt_of_mid_segs[0].y;
             SetRealVertex(vs_return, write_index++, dvs);
             vs_return[write_index].x = round(dvs[2].x);
             vs_return[write_index].y = round(dvs[2].y);
             write_index++;
+   
+            for (j=first_hinge_index+2; j < last_hinge_index-1; j++) {
+               dvs[0].x = mid_pt_of_mid_segs[j-first_hinge_index-2].x;
+               dvs[0].y = mid_pt_of_mid_segs[j-first_hinge_index-2].y;
+               dvs[1].x = (double)(vs[j].x);
+               dvs[1].y = (double)(vs[j].y);
+               dvs[2].x = mid_pt_of_mid_segs[j-first_hinge_index-1].x;
+               dvs[2].y = mid_pt_of_mid_segs[j-first_hinge_index-1].y;
+               SetRealVertex(vs_return, write_index++, dvs);
+               vs_return[write_index].x = round(dvs[2].x);
+               vs_return[write_index].y = round(dvs[2].y);
+               write_index++;
+            }
+            dvs[0].x = mid_pt_of_mid_segs[j-2].x;
+            dvs[0].y = mid_pt_of_mid_segs[j-2].y;
+            dvs[1].x = (double)(vs[last_hinge_index-1].x);
+            dvs[1].y = (double)(vs[last_hinge_index-1].y);
+            dvs[2].x = (double)(vs[last_hinge_index].x);
+            dvs[2].y = (double)(vs[last_hinge_index].y);
+            SetRealVertex(vs_return, write_index++, dvs);
+   
+            free(mid_pt_of_mid_segs);
+         } else {
+            dvs[0].x = (double)(vs[first_hinge_index].x);
+            dvs[0].y = (double)(vs[first_hinge_index].y);
+            dvs[1].x = (double)(vs[first_hinge_index+1].x);
+            dvs[1].y = (double)(vs[first_hinge_index+1].y);
+            dvs[2].x = (double)(vs[first_hinge_index+2].x);
+            dvs[2].y = (double)(vs[first_hinge_index+2].y);
+            SetRealVertex(vs_return, write_index++, dvs);
          }
-         dvs[0].x = mid_pt_of_mid_segs[j-2].x;
-         dvs[0].y = mid_pt_of_mid_segs[j-2].y;
-         dvs[1].x = (double)(vs[last_hinge_index-1].x);
-         dvs[1].y = (double)(vs[last_hinge_index-1].y);
-         dvs[2].x = (double)(vs[last_hinge_index].x);
-         dvs[2].y = (double)(vs[last_hinge_index].y);
-         SetRealVertex(vs_return, write_index++, dvs);
-
-         free(mid_pt_of_mid_segs);
          i = last_hinge_index;
       } else {
          vs_return[write_index].x = vs[i].x;
@@ -835,6 +981,10 @@ int AddObjTickMarks(ObjPtr, nCmdId)
          n = num_cntrl_points;
          vs = cntrl_vs;
          break;
+      case LT_STRUCT_SPLINE:
+         vs = GetStructuredSplinePolyTickMarkVs(&n, ObjPtr, poly_ptr, NULL);
+         vs_allocated = TRUE;
+         break;
       }
       break;
    case OBJ_POLYGON:
@@ -854,6 +1004,10 @@ int AddObjTickMarks(ObjPtr, nCmdId)
          n = num_cntrl_points-1;
          vs = cntrl_vs;
          break;
+      case LT_STRUCT_SPLINE:
+         vs = GetStructuredSplinePolyTickMarkVs(&n, ObjPtr, NULL, polygon_ptr);
+         vs_allocated = TRUE;
+         break;
       }
       break;
    }
