@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.59 2011/05/19 21:31:41 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.60 2011/06/09 15:55:15 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -955,6 +955,15 @@ static TgMenuItemInfo createObjectMenuItemInfo[] = {
    { N_("Get Bounding Box"), NULL,
      N_("Get rectangular bounding boxes of selected objects"), NULL,
      CMDID_GETBOUNDINGBOX },
+   { N_("Create Rectangle From Bounding Box"), NULL,
+     N_("Create a rectangle from the overall bounding box of selected objects"), NULL,
+     CMDID_NEWRECTFROMBOUNDINGBOX },
+   { N_("Create Oval From Bounding Box"), NULL,
+     N_("Create an oval from the overall bounding box of selected objects"), NULL,
+     CMDID_NEWOVALFROMBOUNDINGBOX },
+   { N_("Create Rounded-corner Rectangle From Bounding Box"), NULL,
+     N_("Create a rounded-corner rectangle from the overall bounding box of selected objects"), NULL,
+     CMDID_NEWRCBOXFROMBOUNDINGBOX },
    { N_("Create Pixmap From Selected"), NULL,
      N_("Create an X11 pixmap object from selected objects"), NULL,
      CMDID_CREATEPIXMAPFROMSEL },
