@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/msg.c,v 1.44 2009/10/02 20:14:03 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/msg.c,v 1.49 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -95,6 +95,20 @@ static int		firstCharPos = 0;
 static int		reverseMouseStatusButtons=FALSE;
 static int		gnMsgRows=2;
 
+int ShowOrPrintMsg(buf, use_msgbox)
+   char *buf;
+   int use_msgbox;
+{
+   if (PRTGIF && !cmdLineOpenDisplay) {
+      fprintf(stderr, "%s\n", buf);
+   } else if (use_msgbox) {
+      MsgBox(buf, TOOL_NAME, INFO_MB);
+   } else {
+      Msg(buf);
+   }
+   return FALSE;
+}
+
 int FailAllocMessage()
 {
    fprintf(stderr, "%s\n", TgLoadString(STID_OUT_OF_VMEM_CANNOT_MALLOC));
@@ -1530,7 +1544,7 @@ void Usage(tool_name)
    fprintf(stdout, "\t[-reqcolor] \\\n");
    fprintf(stdout, "\t[-cwo[+sbwarp]] \\\n");
    fprintf(stdout, "\t[-hyper] \\\n");
-   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-a4 | -letter] \\\n");
    fprintf(stdout, "\t[-geometry <geom>] [=<geom>] \\\n");
    fprintf(stdout, "\t[-exec <file>] \\\n");
    fprintf(stdout, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
@@ -1573,7 +1587,7 @@ void Usage(tool_name)
    fprintf(stdout, "\t[-pssetup \"<string>\"] \\\n");
    fprintf(stdout, "\t[-j2p6_cmd \"<command>\"] \\\n");
    fprintf(stdout, "\t[-dontcondense | -condensed] \\\n");
-   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-a4 | -letter] \\\n");
    fprintf(stdout, "\t[-noshowpageineps] \\\n");
    fprintf(stdout, "\t[-quiet] \\\n");
    fprintf(stdout, "\t[-bop_hook \"<string>\"] \\\n");
@@ -1821,6 +1835,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             SetCmdLineCondensedPS();
          } else if (strcmp(*argv, "-a4") == 0) {
             cmdLineA4 = TRUE;
+         } else if (strcmp(*argv, "-letter") == 0) {
+            cmdLineLetter = TRUE;
          } else if (strcmp(*argv, "-noshowpageineps") == 0) {
             cmdLineNoShowPageInEPS = TRUE;
          } else if (strcmp(*argv, "-quiet") == 0) {
@@ -2175,6 +2191,8 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          cmdLineMerge = TRUE;
       } else if (strcmp(argv[i], "-a4") == 0) {
          cmdLineA4 = TRUE;
+      } else if (strcmp(argv[i], "-letter") == 0) {
+         cmdLineLetter = TRUE;
 #ifdef NOT_DEFINED
       } else if (strcmp(argv[i], "-listen") == 0) {
          cmdLineWhiteBoardListenOnly = TRUE;
