@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.8 2009/08/13 01:14:20 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.10 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -128,7 +128,7 @@ char *ReadPath(path_str, dir_str)
    }
    for ( ; *s1 != '\0' && *s1 != ':'; s1++) {
       if (*s1 == '\\') {
-         strcpy(s1, s1+1);
+         UtilStrCpy(s1, s1+1);
       } else {
          *s2++ = *s1;
       }
