@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.4 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.10 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -115,7 +115,7 @@ char *ReadPath(path_str, dir_str)
    }
    for ( ; *s1 != '\0' && *s1 != ':'; s1++) {
       if (*s1 == '\\') {
-         strcpy(s1, s1+1);
+         UtilStrCpy(s1, s1+1);
       } else {
          *s2++ = *s1;
       }
@@ -152,7 +152,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') i++;
+      if (*dir_str != '\0') i++;
    }
    symPath = (char**)malloc(i*sizeof(char*));
    if (symPath == NULL) FailAllocMessage();
@@ -161,7 +161,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') {
+      if (*dir_str != '\0') {
          symPath[i] = (char*)malloc((MAXPATHLENGTH+1)*sizeof(char));
          if (symPath[i] == NULL) FailAllocMessage();
          strcpy(symPath[i], dir_str);
@@ -369,13 +369,13 @@ DspList *DirListing(Path, ExtStr, OtherExtStr)
          if (ext_str != NULL) free(ext_str);
          return NULL;
       }
-#ifdef __EMX__
+#ifdef __CYGWIN__
       if (path[strlen(path)-1] != DIR_SEP) {
          strcat(path, DIR_SEP_STR);
       }
-#else /* ~__EMX__ */
+#else /* ~__CYGWIN__ */
       strcat(path, DIR_SEP_STR);
-#endif /* __EMX__ */
+#endif /* __CYGWIN__ */
    }
 
 #ifdef VMS
