@@ -1,5 +1,5 @@
 /*
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/nkf.c,v 1.2 2009/01/12 05:29:50 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/nkf.c,v 1.3 2011/06/14 02:32:18 william Exp $
  */
 
 /** Network Kanji Filter. (PDS Version)
@@ -28,13 +28,17 @@
 
 #include "tgifdefs.h"
 
-/* for Kconv: _AUTO, _EUC, _SJIS, _JIS */
+#define _AUTO   0
+#define _JIS    1
+#define _EUC    2
+#define _SJIS   3
+
 #include "nkf.e"
 
 #include "setup.e"
 #include "util.e"
 
-int useNKF=FALSE;
+static int useNKF=FALSE;
 
 #ifdef notdef
 static char *CopyRight =
@@ -1605,6 +1609,20 @@ int InitNKF()
    return TRUE;
 }
 
+/* --------------------- Added by Bill Cheng --------------------- */
+
+int Tg_useNKF()
+{
+   return useNKF;
+}
+
+int Tg_do_kconv(i, o, siz)
+   char *i, *o;
+   int siz;
+{
+   return do_kconv(i, o, siz, _EUC, _AUTO);
+}
+
 #endif /* ~_NO_NKF */
 
 /* end */
