@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/obj.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/obj.c,v 1.16 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_OBJ_C_
@@ -103,6 +103,8 @@ void ReplaceObj(old_obj, new_obj)
 void FreeObj(ObjPtr)
    register struct ObjRec *ObjPtr;
 {
+   if (ObjPtr == NULL) return;
+
    if (ObjPtr->ctm != NULL) {
       free(ObjPtr->ctm);
       ObjPtr->ctm = NULL;
@@ -592,10 +594,22 @@ void AdjPolyCache(ObjPtr)
          free(poly_ptr->svlist);
       }
       poly_ptr->svlist = MakeMultiSplinePolyVertex(
-            &(poly_ptr->sn), poly_ptr->smooth,
+            poly_ptr->curved, &(poly_ptr->sn), poly_ptr->smooth,
             drawOrigX, drawOrigY,
             poly_ptr->n, poly_ptr->vlist);
       break;
+   case LT_STRUCT_SPLINE:
+      if (poly_ptr->svlist != NULL) free(poly_ptr->svlist);
+      if (poly_ptr->ssvlist != NULL) free(poly_ptr->ssvlist);
+      if (poly_ptr->ssmooth != NULL) free(poly_ptr->ssmooth);
+      if (poly_ptr->smooth != NULL) free(poly_ptr->smooth);
+      poly_ptr->smooth = NULL;
+      poly_ptr->ssvlist = MakeStructuredSplinePolyVertex(&(poly_ptr->ssn),
+            &(poly_ptr->ssmooth), poly_ptr->n, poly_ptr->vlist);
+      poly_ptr->svlist = MakeMultiSplinePolyVertex(
+            poly_ptr->curved, &(poly_ptr->sn), poly_ptr->ssmooth,
+            drawOrigX, drawOrigY, poly_ptr->ssn, poly_ptr->ssvlist);
+      break;
    case LT_INTSPLINE:
       if (poly_ptr->svlist != NULL) {
          free(poly_ptr->svlist);
@@ -649,12 +663,20 @@ void AdjPolyCache(ObjPtr)
       if (poly_ptr->asvlist != NULL) {
          free(poly_ptr->asvlist);
       }
-      if (poly_ptr->curved != LT_INTSPLINE) {
+      switch (poly_ptr->curved) {
+      case LT_STRAIGHT:
+      case LT_SPLINE:
          v = poly_ptr->vlist;
          n = poly_ptr->n;
-      } else {
+         break;
+      case LT_STRUCT_SPLINE:
+         v = poly_ptr->ssvlist;
+         n = poly_ptr->ssn;
+         break;
+      case LT_INTSPLINE:
          v = poly_ptr->intvlist;
          n = poly_ptr->intn;
+         break;
       }
       v_copy = (IntPoint *)malloc((n+1)*sizeof(IntPoint));
       if (v_copy == NULL) FailAllocMessage();
@@ -692,13 +714,17 @@ void AdjPolyCache(ObjPtr)
       case LT_STRAIGHT:
       case LT_SPLINE:
          poly_ptr->asvlist = MakeMultiSplinePolyVertex(
-               &(poly_ptr->asn), poly_ptr->smooth,
-               drawOrigX, drawOrigY,
-               n, v_copy);
+               poly_ptr->curved, &(poly_ptr->asn), poly_ptr->smooth,
+               drawOrigX, drawOrigY, n, v_copy);
+         break;
+      case LT_STRUCT_SPLINE:
+         poly_ptr->asvlist = MakeMultiSplinePolyVertex(
+               poly_ptr->curved, &(poly_ptr->asn), poly_ptr->ssmooth,
+               drawOrigX, drawOrigY, n, v_copy);
          break;
       case LT_INTSPLINE:
-         poly_ptr->asvlist = MakeSplinePolyVertex(&(poly_ptr->asn),
-               drawOrigX, drawOrigY, n, v_copy);
+         poly_ptr->asvlist = MakeSplinePolyVertex(0, poly_ptr->curved,
+               &(poly_ptr->asn), drawOrigX, drawOrigY, n, v_copy);
          break;
       }
       free(v_copy);
@@ -723,7 +749,7 @@ void AdjPolygonCache(ObjPtr)
          free(polygon_ptr->svlist);
       }
       polygon_ptr->svlist = MakeMultiSplinePolygonVertex(
-            &(polygon_ptr->sn), polygon_ptr->smooth,
+            polygon_ptr->curved, &(polygon_ptr->sn), polygon_ptr->smooth,
             drawOrigX, drawOrigY,
             polygon_ptr->n, polygon_ptr->vlist);
       break;
@@ -732,10 +758,27 @@ void AdjPolygonCache(ObjPtr)
          free(polygon_ptr->svlist);
       }
       polygon_ptr->svlist = MakeMultiSplinePolygonVertex(
-            &(polygon_ptr->sn), polygon_ptr->smooth,
+            polygon_ptr->curved, &(polygon_ptr->sn), polygon_ptr->smooth,
             drawOrigX, drawOrigY,
             polygon_ptr->n, polygon_ptr->vlist);
       break;
+   case LT_STRUCT_SPLINE:
+      if (polygon_ptr->svlist != NULL) free(polygon_ptr->svlist);
+      if (polygon_ptr->ssvlist != NULL) free(polygon_ptr->ssvlist);
+      if (polygon_ptr->ssmooth != NULL) free(polygon_ptr->ssmooth);
+      if (polygon_ptr->smooth != NULL) free(polygon_ptr->smooth);
+#ifdef _TGIF_DBG /* debug, do not translate */
+      TgAssert(polygon_ptr->n >= 7, "NumPts < 7 in AdjPolygonCache()", NULL);
+#endif /* _TGIF_DBG */
+      polygon_ptr->smooth = NULL;
+      polygon_ptr->ssvlist = MakeStructuredSplinePolygonVertex(
+            &(polygon_ptr->ssn), &(polygon_ptr->ssmooth), polygon_ptr->n,
+            polygon_ptr->vlist);
+      polygon_ptr->svlist = MakeMultiSplinePolygonVertex(
+            polygon_ptr->curved, &(polygon_ptr->sn), polygon_ptr->ssmooth,
+            drawOrigX, drawOrigY,
+            polygon_ptr->ssn, polygon_ptr->ssvlist);
+      break;
    case LT_INTSPLINE:
       if (polygon_ptr->svlist != NULL) {
          free(polygon_ptr->svlist);
