@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/page.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/page.c,v 1.15 2011/05/16 16:21:58 william Exp $
  */
 
 #define _INCLUDE_FROM_PAGE_C_
@@ -122,36 +122,79 @@ int DrawAHorizontalTab(page_num, page_ptr, x, y, on_top, skip)
 }
 
 static
-int PageTabsVisible()
+int MaxPageTabsWidth()
 {
-   int i=0, x=0;
-   struct PageRec *page_ptr=NULL;
+   int i=0, total=0;
 
-   x = 1 + (leftend_width<<2);
-   for (i=1, page_ptr=firstPage; page_ptr != NULL;
-         page_ptr=page_ptr->next, i++) {
-      if (i >= pageWindowFirstIndex) {
-         return TRUE;
+   for (i=1; i <= lastPageNum; i++) {
+      total += DrawAHorizontalTab(i, NULL, 0, 0, FALSE, TRUE);
+   }
+   return total;
+}
+
+static
+int PageTabsVisible(pn_last_tab_visible)
+   int *pn_last_tab_visible;
+{
+   int i=0, total=pageWindowW-(scrollBarW<<2)-1;
+   int last_tab_visible=pageWindowFirstIndex;
+
+   for (i=pageWindowFirstIndex; i <= lastPageNum; i++) {
+      total -= DrawAHorizontalTab(i, NULL, 0, 0, FALSE, TRUE);
+      if (total < 0) {
+         break;
       }
+      last_tab_visible = i;
+   }
+   if (pn_last_tab_visible != NULL) {
+      *pn_last_tab_visible = last_tab_visible;
+   }
+   if (curPageNum >= pageWindowFirstIndex &&
+         curPageNum <= last_tab_visible) {
+      return TRUE;
    }
    return FALSE;
 }
 
 static
-void MakePageTabsVisible()
+void MakePageTabsVisible(last_tab_visible)
+   int last_tab_visible;
 {
-   while (pageWindowFirstIndex > 1) {
-      pageWindowFirstIndex--;
-      if (PageTabsVisible()) {
-         return;
+   int total=pageWindowW-(scrollBarW<<2)-1;
+
+   if (MaxPageTabsWidth() <= total) {
+      pageWindowFirstIndex = 1;
+      return;
+   }
+   if (curPageNum < pageWindowFirstIndex) {
+      while (pageWindowFirstIndex > 1) {
+         pageWindowFirstIndex--;
+         if (PageTabsVisible(NULL)) {
+            return;
+         }
       }
+   } else if (curPageNum >= pageWindowFirstIndex &&
+         curPageNum <= last_tab_visible) {
+      return;
+   } else {
+      while (pageWindowFirstIndex <= lastPageNum) {
+         pageWindowFirstIndex++;
+         if (PageTabsVisible(NULL)) {
+            return;
+         }
+      }
+#ifdef _TGIF_DBG /* debug, do not translate */
+      TgAssert(pageWindowFirstIndex > lastPageNum,
+            "pageWindowFirstIndex > lastPageNum in MakePageTabsVisible()", NULL);
+#endif /* _TGIF_DBG */
+      pageWindowFirstIndex = curPageNum;
    }
 }
 
 void RedrawPageWindow()
 {
-   int i, x, y=1;
-   struct PageRec *page_ptr;
+   int i=0, x=0, y=1;
+   struct PageRec *page_ptr=NULL;
 
    XClearArea(mainDisplay, pageWindow, 0, 0, pageWindowW,
          scrollBarW+(brdrW<<1), FALSE);
@@ -159,9 +202,6 @@ void RedrawPageWindow()
          scrollBarW+(brdrW<<1)-1);
    if (pageLayoutMode == PAGE_TILE) return;
 
-   if (!PageTabsVisible()) {
-      MakePageTabsVisible();
-   }
    x = 1 + (leftend_width<<2);
    XSetFont(mainDisplay, defaultGC, rulerFontPtr->fid);
    for (i=1, page_ptr=firstPage; page_ptr != NULL;
@@ -211,6 +251,17 @@ void RedrawPageWindow()
    XSetTSOrigin(mainDisplay, rasterGC, 0, 0);
 }
 
+void MakeCurPageTabVisible()
+{
+   int last_tab_visible=(-1);
+
+   if (pageLayoutMode == PAGE_TILE) return;
+
+   if (!PageTabsVisible(&last_tab_visible)) {
+      MakePageTabsVisible(last_tab_visible);
+   }
+}
+
 void GotoPageNum(new_page_num)
    int new_page_num;
 {
@@ -242,6 +293,7 @@ void GotoPageNum(new_page_num)
       curPage->zoom_scale = zoomScale;
       curPage->zoomed_in = zoomedIn;
    }
+   MakeCurPageTabVisible();
 }
 
 static int curChoiceBeforePageChange=NOTHING;
@@ -279,6 +331,7 @@ void SetCurPage(page_number)
       ClearAndRedrawDrawWindow();
    }
    RedrawTitleWindow();
+   MakeCurPageTabVisible();
    ShowPage();
    SetCurChoice(curChoiceBeforePageChange);
    if (gstWBInfo.do_whiteboard && topObj != NULL) {
@@ -973,6 +1026,7 @@ void AddPage(AfterCurPage, papcai)
    ClearAndRedrawDrawWindow();
    RedrawTitleWindow();
    SetFileModified(TRUE);
+   MakeCurPageTabVisible();
    ShowPage();
    if (papcai == NULL && !cmdLineMerge) {
       if ((attr_ptr=FindAttrWithName(tgifObj, "template=", NULL)) != NULL) {
@@ -1098,6 +1152,7 @@ void DeleteCurPage()
 
    ClearAndRedrawDrawWindow();
    RedrawTitleWindow();
+   MakeCurPageTabVisible();
    ShowPage();
 
    sprintf(gszMsgBox, TgLoadString(STID_GIVEN_PAGE_NUM_DELETED), n);
@@ -1263,6 +1318,37 @@ int GetDimension(Spec, nNonPosReturnOK, pnNumPixels)
    return TRUE;
 }
 
+int GetDimensionInDouble(Spec, nNonPosReturnOK, pd_num_pixels)
+   char *Spec;
+   int nNonPosReturnOK;
+   double *pd_num_pixels;
+{
+   char *c_ptr=NULL;
+   double dval=(double)0;
+
+   /* do not translate -- program constants */
+   if ((c_ptr=strstr(Spec, "in")) != NULL ||
+         (c_ptr=strstr(Spec, "In")) != NULL ||
+         (c_ptr=strstr(Spec, "IN")) != NULL) {
+      *c_ptr = '\0';
+      dval = atof(Spec) * ((double)PIX_PER_INCH);
+      *pd_num_pixels = dval;
+      if (!nNonPosReturnOK && dval < (double)0) return FALSE;
+   } else if ((c_ptr=strstr(Spec, "cm")) != NULL ||
+         (c_ptr=strstr(Spec, "Cm")) != NULL ||
+         (c_ptr=strstr(Spec, "CM")) != NULL) {
+      *c_ptr = '\0';
+      dval = atof(Spec) * ((double)ONE_CM);
+      *pd_num_pixels = dval;
+      if (!nNonPosReturnOK && dval < (double)0) return FALSE;
+   } else if (sscanf(Spec, "%lf", pd_num_pixels) != 1) {
+      return FALSE;
+   } else {
+      if (!nNonPosReturnOK && dval < (double)0) return FALSE;
+   }
+   return TRUE;
+}
+
 int SetPaperSize(Spec)
    char *Spec;
 {
@@ -1596,6 +1682,7 @@ void PrintPages()
    case PNG_FILE:
    case JPEG_FILE:
    case PPM_FILE:
+   case SVG_FILE:
       sprintf(gszMsgBox, TgLoadString(STID_CANT_PRINT_PAGES_IN_FORMAT),
             GetExportName(whereToPrint, EXPORT_THIS));
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -2041,7 +2128,7 @@ void PageEventHandler(input)
 void CleanUpPage()
 {
    DelAllPages();
-   if (leftend_bits) { }
+   if (leftend_bits == NULL) { }
 }
 
 static PageInfo stPageInfo;
