@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/polygon.c,v 1.40 2011/05/16 16:21:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/polygon.c,v 1.41 2011/06/09 15:55:15 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_POLYGON_C_
@@ -1971,13 +1971,14 @@ void AddToLastPoint(XOff, YOff)
 void GetBoundingBox()
 {
    struct SelRec *sel_ptr, *tmp_top_sel=NULL, *tmp_bot_sel=NULL;
-   int num_created=0;
+   int num_created=0, saved_cur_spline=curSpline;
 
    if (topSel == NULL) {
       MsgBox(TgLoadCachedString(CSTID_NO_OBJ_SELECTED), TOOL_NAME, INFO_MB);
       return;
    }
    if (curChoice == VERTEXMODE) SetCurChoice(NOTHING);
+   curSpline = LT_STRAIGHT;
 
    tmp_top_sel = tmp_bot_sel = NULL;
 
@@ -2026,6 +2027,7 @@ void GetBoundingBox()
       num_created++;
    }
    EndCompositeCmd();
+   curSpline = saved_cur_spline;
 
    RemoveAllSel();
    topSel = tmp_top_sel;
