@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.14 2009/11/09 05:20:10 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.16 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_PS_C_
@@ -131,7 +131,7 @@ void PSUseColorImage()
 {
    if (preDumpSetup && !psUseColorImage) {
       psUseColorImage = TRUE;
-      psDictCount += 11;
+      psDictCount += 13;
    }
 }
 
@@ -606,6 +606,16 @@ static char *psColorImageMacro[] = {
   "",
   "/tgifsetpix { tgifxpmdict begin pixels 3 1 roll putinterval end } def",
   "",
+  "/tgifsetppmtruetranspix",
+  " { tgifxpmdict begin",
+  "      3 array /transpixels exch def",
+  "      transpixels 2 3 -1 roll put",
+  "      transpixels 1 3 -1 roll put",
+  "      transpixels 0 3 -1 roll put",
+  "      /curpix 3 array def",
+  "   end",
+  " } def",
+  "",
   "/tgifcolorspot",
   " { tgifxpmdict begin",
   "      /index exch def",
@@ -617,13 +627,6 @@ static char *psColorImageMacro[] = {
   "   end",
   " } def",
   "",
-  "/tgifnewcolorspot",
-  " { tgifxpmdict begin",
-  "      /index exch def",
-  "      pixels index 3 mul 3 getinterval aload pop setrgbcolor",
-  "   end",
-  " } def",
-  "",
   "/tgifcolordict 4 dict def",
   "",
   "/colorimage where",
@@ -637,8 +640,8 @@ static char *psColorImageMacro[] = {
   "        /y 0 def",
   "        1 1 ih",
   "         { pop 1 1 iw",
-  "            { pop currentfile",
-  "              tgifbwpicstr readhexstring pop 0 get tgifnewcolorspot",
+  "            { pop",
+  "              3 { currentfile tgifbwpicstr readhexstring pop 0 get 255 div } repeat setrgbcolor",
   "              newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
   "              closepath fill",
   "              /x x 1 add def",
@@ -681,6 +684,42 @@ static char *psColorImageMacro[] = {
   "   end",
   " } def",
   "",
+  "/tgiftransppmtruecolorspot",
+  " { tgifxpmdict begin",
+  "      /count 0 def",
+  "      curpix 2 3 -1 roll put",
+  "      curpix 1 3 -1 roll put",
+  "      curpix 0 3 -1 roll put",
+  "      curpix 0 get transpixels 0 get ne { /count count 1 add def } if",
+  "      curpix 1 get transpixels 1 get ne { /count count 1 add def } if",
+  "      curpix 2 get transpixels 2 get ne { /count count 1 add def } if",
+  "      count 0 eq { false } { curpix 0 get 255 div curpix 1 get 255 div curpix 2 get 255 div setrgbcolor true } ifelse",
+  "   end",
+  " } def",
+  "",
+  "/tgiftransppmtruecolorimage",
+  " { tgifcolordict begin",
+  "      pop pop pop pop pop",
+  "      /ih exch def",
+  "      /iw exch def",
+  "      /x 0 def",
+  "      /y 0 def",
+  "      1 1 ih",
+  "       { pop 1 1 iw",
+  "          { pop",
+  "            3 { currentfile tgifbwpicstr readhexstring pop 0 get } repeat tgiftransppmtruecolorspot",
+  "            { newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
+  "              closepath fill",
+  "            }",
+  "            if",
+  "            /x x 1 add def",
+  "          } for",
+  "         /y y 1 add def",
+  "         /x 0 def",
+  "       } for",
+  "   end",
+  " } def",
+  "",
   NULL
 };
 
