@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.16 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_PS_C_
@@ -25,9 +25,10 @@
 #include "file.e"
 #include "msg.e"
 #include "ps.e"
+#include "raster.e"
 #include "util.e"
 
-#define DEF_PS_DICT_COUNT 2
+#define DEF_PS_DICT_COUNT 3
 
 int preDumpSetup=FALSE;
 int needsTgifOrigCTM=TRUE;
@@ -117,7 +118,7 @@ void PSUseColorImage()
 {
    if (preDumpSetup && !psUseColorImage) {
       psUseColorImage = TRUE;
-      psDictCount += 11;
+      psDictCount += 13;
    }
 }
 
@@ -592,6 +593,16 @@ static char *psColorImageMacro[] = {
   "",
   "/tgifsetpix { tgifxpmdict begin pixels 3 1 roll putinterval end } def",
   "",
+  "/tgifsetppmtruetranspix",
+  " { tgifxpmdict begin",
+  "      3 array /transpixels exch def",
+  "      transpixels 2 3 -1 roll put",
+  "      transpixels 1 3 -1 roll put",
+  "      transpixels 0 3 -1 roll put",
+  "      /curpix 3 array def",
+  "   end",
+  " } def",
+  "",
   "/tgifcolorspot",
   " { tgifxpmdict begin",
   "      /index exch def",
@@ -603,13 +614,6 @@ static char *psColorImageMacro[] = {
   "   end",
   " } def",
   "",
-  "/tgifnewcolorspot",
-  " { tgifxpmdict begin",
-  "      /index exch def",
-  "      pixels index 3 mul 3 getinterval aload pop setrgbcolor",
-  "   end",
-  " } def",
-  "",
   "/tgifcolordict 4 dict def",
   "",
   "/colorimage where",
@@ -623,8 +627,8 @@ static char *psColorImageMacro[] = {
   "        /y 0 def",
   "        1 1 ih",
   "         { pop 1 1 iw",
-  "            { pop currentfile",
-  "              tgifbwpicstr readhexstring pop 0 get tgifnewcolorspot",
+  "            { pop",
+  "              3 { currentfile tgifbwpicstr readhexstring pop 0 get 255 div } repeat setrgbcolor",
   "              newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
   "              closepath fill",
   "              /x x 1 add def",
@@ -667,19 +671,56 @@ static char *psColorImageMacro[] = {
   "   end",
   " } def",
   "",
+  "/tgiftransppmtruecolorspot",
+  " { tgifxpmdict begin",
+  "      /count 0 def",
+  "      curpix 2 3 -1 roll put",
+  "      curpix 1 3 -1 roll put",
+  "      curpix 0 3 -1 roll put",
+  "      curpix 0 get transpixels 0 get ne { /count count 1 add def } if",
+  "      curpix 1 get transpixels 1 get ne { /count count 1 add def } if",
+  "      curpix 2 get transpixels 2 get ne { /count count 1 add def } if",
+  "      count 0 eq { false } { curpix 0 get 255 div curpix 1 get 255 div curpix 2 get 255 div setrgbcolor true } ifelse",
+  "   end",
+  " } def",
+  "",
+  "/tgiftransppmtruecolorimage",
+  " { tgifcolordict begin",
+  "      pop pop pop pop pop",
+  "      /ih exch def",
+  "      /iw exch def",
+  "      /x 0 def",
+  "      /y 0 def",
+  "      1 1 ih",
+  "       { pop 1 1 iw",
+  "          { pop",
+  "            3 { currentfile tgifbwpicstr readhexstring pop 0 get } repeat tgiftransppmtruecolorspot",
+  "            { newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
+  "              closepath fill",
+  "            }",
+  "            if",
+  "            /x x 1 add def",
+  "          } for",
+  "         /y y 1 add def",
+  "         /x 0 def",
+  "       } for",
+  "   end",
+  " } def",
+  "",
   NULL
 };
 
 static char *psColorPatternMacro[] = {
-  "/tgifpatdict 10 dict def",
+  "/tgifpatdict 12 dict def",
   "",
   "/tgifpatbyte",
   " { currentdict /retstr get exch",
-  "   pat i cellsz mod get put",
+  "   bytesperrow mul",
+  "   pat i cellh mod bytesperrow mul bytesperrow getinterval putinterval",
   " } def",
   "",
   "/tgifpatproc",
-  " { 0 1 widthlim {tgifpatbyte} for retstr",
+  " { 0 1 cells_wd 1 sub {tgifpatbyte} for retstr",
   "   /i i 1 add def",
   " } def",
   "",
@@ -689,11 +730,13 @@ static char *psColorPatternMacro[] = {
   "      /w exch def",
   "      /lty exch def",
   "      /ltx exch def",
-  "      /cellsz exch def",
+  "      /cellh exch def",
+  "      /cellw exch def",
   "      /pat exch def",
   "",
-  "      /widthlim w cellsz div cvi 1 sub def",
-  "      /retstr widthlim 1 add string def",
+  "      /cells_wd w cellw div cvi def",
+  "      /bytesperrow cellw 8 div ceiling cvi def",
+  "      /retstr cells_wd bytesperrow mul string def",
   "      /i 0 def",
   "",
   "      tgiforigctm setmatrix",
@@ -706,39 +749,60 @@ static char *psColorPatternMacro[] = {
   NULL
 };
 
-static char *psPatternMacro[] = {
-  "/pat3 <8000000008000000> def",
-  "/pat4 <8800000022000000> def",
-  "/pat5 <8800220088002200> def",
-  "/pat6 <8822882288228822> def",
-  "/pat7 <aa55aa55aa55aa55> def",
-  "/pat8 <77dd77dd77dd77dd> def",
-  "/pat9 <77ffddff77ffddff> def",
-  "/pat10 <77ffffff77ffffff> def",
-  "/pat11 <7fffffff7fffffff> def",
-  "/pat12 <8040200002040800> def",
-  "/pat13 <40a00000040a0000> def",
-  "/pat14 <ff888888ff888888> def",
-  "/pat15 <ff808080ff080808> def",
-  "/pat16 <f87422478f172271> def",
-  "/pat17 <038448300c020101> def",
-  "/pat18 <081c22c180010204> def",
-  "/pat19 <8080413e080814e3> def",
-  "/pat20 <8040201008040201> def",
-  "/pat21 <8844221188442211> def",
-  "/pat22 <77bbddee77bbddee> def",
-  "/pat23 <c1e070381c0e0783> def",
-  "/pat24 <7fbfdfeff7fbfdfe> def",
-  "/pat25 <3e1f8fc7e3f1f87c> def",
-  "/pat26 <0102040810204080> def",
-  "/pat27 <1122448811224488> def",
-  "/pat28 <eeddbb77eeddbb77> def",
-  "/pat29 <83070e1c3870e0c1> def",
-  "/pat30 <fefdfbf7efdfbf7f> def",
-  "/pat31 <7cf8f1e3c78f1f3e> def",
-  "",
-  NULL
-};
+/*
+ * No longer needed.
+ * Now psDynamicPrintPatternDef() generates these from pixmap data.
+ *
+ * static char *psPatternMacro[] = {
+ *   "/pat3 <8000000008000000> def",
+ *   "/pat4 <8800000022000000> def",
+ *   "/pat5 <8800220088002200> def",
+ *   "/pat6 <8822882288228822> def",
+ *   "/pat7 <aa55aa55aa55aa55> def",
+ *   "/pat8 <77dd77dd77dd77dd> def",
+ *   "/pat9 <77ffddff77ffddff> def",
+ *   "/pat10 <77ffffffddffffff> def",
+ *   "/pat11 <7ffffffff7ffffff> def",
+ *   "/pat12 <8040200002040800> def",
+ *   "/pat13 <40a00000040a0000> def",
+ *   "/pat14 <ff888888ff888888> def",
+ *   "/pat15 <ff808080ff080808> def",
+ *   "/pat16 <f87422478f172271> def",
+ *   "/pat17 <038448300c020101> def",
+ *   "/pat18 <081c22c180010204> def",
+ *   "/pat19 <8080413e080814e3> def",
+ *   "/pat20 <8040201008040201> def",
+ *   "/pat21 <8844221188442211> def",
+ *   "/pat22 <77bbddee77bbddee> def",
+ *   "/pat23 <c1e070381c0e0783> def",
+ *   "/pat24 <7fbfdfeff7fbfdfe> def",
+ *   "/pat25 <3e1f8fc7e3f1f87c> def",
+ *   "/pat26 <0102040810204080> def",
+ *   "/pat27 <1122448811224488> def",
+ *   "/pat28 <eeddbb77eeddbb77> def",
+ *   "/pat29 <83070e1c3870e0c1> def",
+ *   "/pat30 <fefdfbf7efdfbf7f> def",
+ *   "/pat31 <7cf8f1e3c78f1f3e> def",
+ *   "",
+ *   NULL
+ * };
+ */
+
+void psDynamicPrintPatternDef(FP, index)
+   FILE *FP;
+   int index;
+{
+   int b=0, numbytes=0, need_to_free=FALSE;
+   unsigned char *bytes=NULL;
+
+   bytes = (unsigned char *)GetPatternBytes(index, &numbytes, &need_to_free);
+   fprintf(FP, "/pat%d <", index);
+   for (b=0; b < numbytes; b++) {
+      fprintf(FP, "%02hhx", ByteReverse(bytes[b]));
+   }
+   fprintf(FP, "> def\n");
+   if (need_to_free) UtilFree((char*)bytes);
+}
 
 static char *psCenterTextMacro[] = {
   "/TGCTX { dup stringwidth pop 2 div neg 0 rmoveto } def",
@@ -754,6 +818,7 @@ static char *psMiniLinesMacro[] = {
   " { exch dup 3 1 roll exch dup 3 1 roll lt { pop } { exch pop } ifelse",
   " } def",
   "/TGSW { stringwidth pop } def",
+  "/tgifstrw 0 def",
   "",
   NULL
 };
@@ -981,9 +1046,15 @@ void DumpPSMacro(FP)
       }
    }
    if (psUsePattern) {
-      for (i=0; psPatternMacro[i] != NULL; i++) {
-         fprintf(FP, "%s\n", psPatternMacro[i]);
+      /*
+       * for (i=0; psPatternMacro[i] != NULL; i++) {
+       *    fprintf(FP, "%s\n", psPatternMacro[i]);
+       * }
+       */
+      for (i=3; i < 32; i++) {
+         psDynamicPrintPatternDef(FP, i);
       }
+      fprintf(FP, "\n");
    }
    if (psUseCenterText) {
       for (i=0; psCenterTextMacro[i] != NULL; i++) {
