@@ -39,10 +39,13 @@ typedef struct
 } Config;
 
 Config config;
+FILE  *fp;
+char   mcastproxy_config_file[255];
 
 typedef void (*sighandler_t)(int);
 
 int  read_config_file( FILE *, Config * );
+void re_read_config_file( int );
 int  create_ucast_socket( Config * );
 int  create_mcast_socket( Config *, struct sockaddr_in *);
 int  convert_addr( Config *, struct sockaddr_in *, struct sockaddr_in ** );
@@ -51,21 +54,21 @@ void showHosts( int );
 
 int main( int argc, char * argv[] )
 {
-    FILE  *fp;
     FILE  *pidfile;
     struct sockaddr_in group_addr;
     struct sockaddr_in * addr_list;
     fd_set readfds;
     int    mcast_sock, ucast_sock, byread, bysent, i;
     char   pack_buf[MAX_PACK_SIZE];
-    char   mcastproxy_config_file[255];
     char   mcastproxy_pid_file[255];
     char  *homedir;
     unsigned char flags;
 
     signal( SIGUSR1, showHosts );
+    signal( SIGHUP, re_read_config_file );
 
-    if( ( homedir = getenv( "HOME" ) ) != NULL ){
+    if( ( homedir = getenv( "HOME" ) ) != NULL )
+    {
          sprintf( mcastproxy_pid_file, "%s/.tgwb/mcastproxy.pid", homedir );
          if( ( pidfile = fopen( mcastproxy_pid_file, "w" ) ) == NULL )
          {
@@ -268,6 +271,20 @@ int read_config_file( FILE * fp, Config * config )
     return 0;
 }
 
+void re_read_config_file( int signo )
+{
+    if( ( fp = fopen( mcastproxy_config_file, "r" ) ) == NULL )
+    {
+         fprintf( stderr, "\n[mcastproxy Error] Opening configuration file '%s'\n", mcastproxy_config_file );
+         exit( MP_ERROR_OPENCONFFILE );
+    }
+    if( read_config_file( fp, &config ) < 0 )
+    {
+        fprintf( stderr, "\n[mcastproxy Error] Invalid new configuration file." );
+        exit( MP_ERROR_INVALIDCONFFILE );
+    }
+}
+
 int create_ucast_socket( Config * config )
 {
     int    sckd;
