@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/scroll.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/scroll.c,v 1.8 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_SCROLL_C_
@@ -112,11 +112,22 @@ int TgPressButtonLoop(dpy, win, pbbox, psbci)
       initial_timeout = FALSE;
 
       if (status < 0) {
-         XUngrabPointer(dpy, CurrentTime);
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-               "TgPressButtonLoop()");
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         break;
+         if (errno == EINTR) {
+            /* interrupted by a system call, do it again */
+            if (TgAnyButtonDown(dpy, win)) {
+               need_to_scroll_once = FALSE;
+               if ((psbci->pf_scroll_btn_callback)(psbci->pv_userdata)) {
+                  XUngrabPointer(dpy, CurrentTime);
+                  return TRUE;
+               }
+            }
+         } else {
+            XUngrabPointer(dpy, CurrentTime);
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                  "TgPressButtonLoop()");
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            break;
+         }
       } else if (status == 0 && TgAnyButtonDown(dpy, win)) {
          need_to_scroll_once = FALSE;
          if ((psbci->pf_scroll_btn_callback)(psbci->pv_userdata)) {
@@ -1504,7 +1515,7 @@ void GetOrigin(pn_x, pn_y, pn_zoom_scale, pn_zoomed_in)
 
 void CleanUpScrolls()
 {
-   if (scrl_up_bits) { }
+   if (scrl_up_bits == NULL) { }
 }
 
 void UpdScrollWinWH()
