@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/select.c,v 1.24 2009/09/28 17:34:30 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/select.c,v 1.26 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
@@ -388,6 +388,9 @@ struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
          if (strcmp(attr_ptr->attr_name.s, AttrName) == 0) {
             if (GetTextObjFirstStrSeg(attr_ptr->obj)->color == colorIndex) {
                break;
+            } else if (GetTextObjFirstStrSeg(attr_ptr->obj)->color ==
+                  GetTextObjFirstStrSeg(found_attr->obj)->color) {
+               /* same color as found_attr's color, so don't increase count */
             } else {
                count++;
             }
@@ -396,6 +399,9 @@ struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
          if (strcmp(attr_ptr->attr_value.s, AttrName) == 0) {
             if (GetTextObjFirstStrSeg(attr_ptr->obj)->color == colorIndex) {
                break;
+            } else if (GetTextObjFirstStrSeg(attr_ptr->obj)->color ==
+                  GetTextObjFirstStrSeg(found_attr->obj)->color) {
+               /* same color as found_attr's color, so don't increase count */
             } else {
                count++;
             }
