@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.82 2011/05/19 21:59:45 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.86 2011/06/02 17:35:50 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -839,17 +839,21 @@ void HandleSelectionRequest(input)
 
       if (gSetCutBufferInfo.utf8_valid) {
          DebugSelectRequest(property, target, 0, "COMPOUND_TEXT for UTF8");
+#ifndef _NO_XUTF8FUNCTIONS
          Xutf8TextListToTextProperty(mainDisplay,
                &(gSetCutBufferInfo.utf8_dyn_str.s), 1,
                XCompoundTextStyle, &text_property);
+#endif /* ~_NO_XUTF8FUNCTIONS */
       } else if (gSetCutBufferInfo.text_valid) {
          DebugSelectRequest(property, target, 0, "COMPOUND_TEXT not UTF8");
          XmbTextListToTextProperty(mainDisplay,
                &(gSetCutBufferInfo.text_dyn_str.s), 1,
                XCompoundTextStyle, &text_property);
       } else {
+         char *psz_buf=(char*)empty_buf;
+
          DebugSelectRequest(property, target, 0, "COMPOUND_TEXT (none)");
-         XmbTextListToTextProperty(mainDisplay, (char**)(&empty_buf), 1,
+         XmbTextListToTextProperty(mainDisplay, &psz_buf, 1,
                XCompoundTextStyle, &text_property);
       }
       XChangeProperty(mainDisplay, requestor, property, target,
