@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.71 2009/11/08 00:25:03 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.86 2011/06/02 17:35:50 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -86,6 +86,7 @@
 #include "xbitmap.e"
 #include "xpixmap.e"
 #include "xprtfltr.e"
+#include "z_intrf.e"
 
 #include "tgificon.xbm"
 
@@ -120,6 +121,7 @@
 #define CHAT_WINDOW_H 40
 
 int	debugNoPointerGrab=FALSE;
+int	debugCopyPaste=FALSE;
 
 unsigned int	mainWinW = 0;
 unsigned int	mainWinH = 0;
@@ -686,8 +688,7 @@ void HandleSelectionClear(input)
    selection = input->xselectionclear.selection;
    time = input->xselectionclear.time;
 
-#ifdef NOT_DEFINED
-   {
+   if (debugCopyPaste) {
       static int id=0;
       char *atom_name=NULL;
 
@@ -697,7 +698,6 @@ void HandleSelectionClear(input)
       printf("%1d: Selection name is '%s'.\n", id, atom_name);
       XFree(atom_name);
    }
-#endif /* NOT_DEFINED */
    if (startSelectionOwnershipTimeValid) {
       endSelectionOwnershipTime = time;
       endSelectionOwnershipTimeValid = TRUE;
@@ -705,6 +705,35 @@ void HandleSelectionClear(input)
    ClearSelection();
 }
 
+static
+void DebugSelectRequest(property, target, level, msg)
+   Atom property, target;
+   int level;
+   char *msg;
+{
+   if (debugCopyPaste > level) {
+      /*
+       * After tgif does a Copy, tgif will get SelectionRequest at
+       *         a regular interval
+       */
+      static int id=0;
+      char *atom_name=NULL;
+
+      if (msg == NULL) {
+         printf("\n%1d: SelectionRequest.\n", ++id);
+      } else {
+         printf("\n%1d: SelectionRequest (%s).\n", ++id, msg);
+      }
+      atom_name = XGetAtomName(mainDisplay, property);
+      printf("%1d: Property name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+
+      atom_name = XGetAtomName(mainDisplay, target);
+      printf("%1d: Target name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+   }
+}
+
 void HandleSelectionRequest(input)
    XEvent *input;
 {
@@ -712,33 +741,17 @@ void HandleSelectionRequest(input)
    Atom property=(Atom)0, target=(Atom)0, selection=(Atom)0;
    Time time=(Time)0;
    XEvent sent_ev;
+   unsigned char empty_buf[1];
 
+   *empty_buf = '\0';
    requestor = input->xselectionrequest.requestor;
    selection = input->xselectionrequest.selection;
    target = input->xselectionrequest.target;
    property = input->xselectionrequest.property;
    time = input->xselectionrequest.time;
 
-#ifdef NOT_DEFINED
-   {
-      /*
-       * After tgif does a Copy, tgif will get SelectionRequest at
-       *         a regular interval
-       */
-      static int id=0;
-      char *atom_name=NULL;
-
-      printf("\n%1d: SelectionRequest.\n", ++id);
-
-      atom_name = XGetAtomName(mainDisplay, property);
-      printf("%1d: Property name is '%s'.\n", id, atom_name);
-      XFree(atom_name);
-
-      atom_name = XGetAtomName(mainDisplay, target);
-      printf("%1d: Target name is '%s'.\n", id, atom_name);
-      XFree(atom_name);
-   }
-#endif /* NOT_DEFINED */
+   /* do not translate -- program constants */
+   DebugSelectRequest(property, target, 1, "function entry point");
    memset(&sent_ev, 0, sizeof(XEvent));
 
    sent_ev.xselection.type = SelectionNotify;
@@ -755,35 +768,79 @@ void HandleSelectionRequest(input)
        *         object copied into (i.e., what are the possobiel targets)?
        */
       Atom targets[5];
+      int num_targets=0;
 
-      targets[0] = XA_STRING;
-      targets[1] = compoundTextAtom;
-      targets[2] = tgifProtocolAtom;
-      targets[3] = textAtom;
-      targets[4] = utf8StringAtom;
+      DebugSelectRequest(property, target, 0, "TARGETS");
+      if (gSetCutBufferInfo.tgif_valid) {
+         targets[num_targets++] = tgifProtocolAtom;
+      }
+      if (gSetCutBufferInfo.utf8_valid) {
+         targets[num_targets++] = utf8StringAtom;
+      }
+      if (gSetCutBufferInfo.text_valid) {
+         targets[num_targets++] = compoundTextAtom;
+         targets[num_targets++] = textAtom;
+         targets[num_targets++] = XA_STRING;
+      }
       XChangeProperty(mainDisplay, requestor, property, target, 32,
-            PropModeReplace, (unsigned char *)targets, 5);
+            PropModeReplace, (unsigned char *)targets, num_targets);
       sent_ev.xselection.property = property;
    } else if (target == XA_STRING || target == textAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "XA_STRING or TEXT");
+      if (gSetCutBufferInfo.text_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.text_dyn_str.s),
+               gSetCutBufferInfo.text_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == utf8StringAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "UTF8_STRING");
+      if (gSetCutBufferInfo.utf8_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.utf8_dyn_str.s),
+               gSetCutBufferInfo.utf8_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == tgifProtocolAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "_TGIF_PROTOCOL_ATOM");
+      if (gSetCutBufferInfo.tgif_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.tgif_dyn_str.s),
+               gSetCutBufferInfo.tgif_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == compoundTextAtom) {
       XTextProperty text_property;
 
-      if (cutBufferIsUTF8) {
-         Xutf8TextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+      if (gSetCutBufferInfo.utf8_valid) {
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT for UTF8");
+#ifndef _NO_XUTF8FUNCTIONS
+         Xutf8TextListToTextProperty(mainDisplay,
+               &(gSetCutBufferInfo.utf8_dyn_str.s), 1,
+               XCompoundTextStyle, &text_property);
+#endif /* ~_NO_XUTF8FUNCTIONS */
+      } else if (gSetCutBufferInfo.text_valid) {
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT not UTF8");
+         XmbTextListToTextProperty(mainDisplay,
+               &(gSetCutBufferInfo.text_dyn_str.s), 1,
                XCompoundTextStyle, &text_property);
       } else {
-         XmbTextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+         char *psz_buf=(char*)empty_buf;
+
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT (none)");
+         XmbTextListToTextProperty(mainDisplay, &psz_buf, 1,
                XCompoundTextStyle, &text_property);
       }
       XChangeProperty(mainDisplay, requestor, property, target,
@@ -896,7 +953,7 @@ int mainWinEventHandler(input)
                   TranslateKeys(buf, &key_sym);
                }
                /*
-                * This is probabl not needed.
+                * This is probably not needed.
                 *
                 * ClearCopyUTF8Info(); 
                 * if (textHighlight) {
@@ -1493,6 +1550,9 @@ void Setup()
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"InitialPaperSize")) != NULL) {
       paper_size_set = SetPaperSize(c_ptr);
    }
+   if (cmdLineLetter) {
+      paper_size_set = SetPaperSize("letter");
+   }
    pageStyle = PORTRAIT;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PageStyleLandscape")) !=
          NULL && UtilStrICmp("true", c_ptr) == 0) {
@@ -1723,9 +1783,19 @@ void Setup()
          mainVisual->class==StaticGray) ? FALSE : TRUE;
    fullTrueColorMode = FALSE;
    if (colorDisplay && mainVisual->class == TrueColor) {
-      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
-            NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      if (HasZlibSupport()) {
+         /* starting with tgif-4.2.3, this is the default */
          fullTrueColorMode = TRUE;
+#ifdef _TGIF_DBG
+      } else {
+         fprintf(stderr, TgLoadString(STID_CANNOT_SET_XDEF_TRUE_NO_ZLIB),
+               TOOL_NAME, "EnableTrueColorImages", TOOL_NAME);
+         fprintf(stderr, "\n");
+#endif /* _TGIF_DBG */
+      }
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
+            NULL && UtilStrICmp(c_ptr, "false") == 0) {
+         fullTrueColorMode = FALSE;
       }
    }
    noAntiAlias = TRUE;
