@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shape.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shape.c,v 1.8 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_SHAPE_C_
@@ -663,8 +663,9 @@ void HighLightCreateShape(end_x, end_y, generate, pn_need_to_draw)
                   gnNumVs, gpVertices);
             gnNumSplineVs = gnNumVs;
          } else {
-            gpRubberVertices = MakeMultiSplinePolygonVertex(&gnNumSplineVs,
-                  gpnSmooth, drawOrigX, drawOrigY, gnNumVs, gpVertices);
+            gpRubberVertices = MakeMultiSplinePolygonVertex(LT_SPLINE,
+                  &gnNumSplineVs, gpnSmooth, drawOrigX, drawOrigY, gnNumVs,
+                  gpVertices);
          }
          if (gpRubberVertices == NULL) FailAllocMessage();
          drawOrigX = saved_draw_orig_x;
@@ -1003,10 +1004,11 @@ void ContinueCreateShape(orig_x, orig_y)
    BeginIntervalRulers(orig_x, orig_y, orig_x, orig_y);
    DoCursor(orig_x, orig_y, 0, 0, START_SHOW);
 
-   XGrabPointer(mainDisplay, drawWindow, False,
-         PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
-         GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
-
+   if (!debugNoPointerGrab) {
+      XGrabPointer(mainDisplay, drawWindow, False,
+            PointerMotionMask | ButtonReleaseMask | ButtonReleaseMask,
+            GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
+   }
    if (!StartCreateShape(orig_x, orig_y)) {
       Msg(TgLoadString(STID_CREATE_A_SHAPE_FAILED));
       return;
