@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shortcut.c,v 1.46 2011/05/16 16:21:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shortcut.c,v 1.47 2011/06/09 16:11:42 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -481,6 +481,9 @@ static struct ShortCutRec shortCutXlateTbl[] = {
                CMDID_SETFLOODREPLACECOLORTHRESH },
    { '\0', 0, "RemoveTransparentPixel()", 0, CMDID_REMOVETRANSPIXEL },
    { '\0', 0, "ReplaceColorWithTrans()", 0, CMDID_REPLACECOLORWITHTRANS },
+   { '\0', 0, "MakeBoxObjFromBoundingBox()", 0, CMDID_NEWRECTFROMBOUNDINGBOX },
+   { '\0', 0, "MakeRCBoxObjFromBoundingBox()", 0, CMDID_NEWRCBOXFROMBOUNDINGBOX },
+   { '\0', 0, "MakeOvalObjFromBoundingBox()", 0, CMDID_NEWOVALFROMBOUNDINGBOX },
    { '\0', 0, "", 0, 0 }
 };
 
