@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/stk.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/stk.c,v 1.7 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_STK_C_
@@ -151,6 +151,8 @@ struct StkRec *SaveFileInfo()
    stk_ptr->rcb_radius = rcbRadius;
    stk_ptr->underline_on = curUnderlineOn;
    stk_ptr->underline_y_offset = curUnderlineYOffset;
+   stk_ptr->overline_on = curOverlineOn;
+   stk_ptr->overline_y_offset = curOverlineYOffset;
    stk_ptr->first_file_attr = tgifObj->fattr;
    stk_ptr->last_file_attr = tgifObj->lattr;
 
@@ -400,6 +402,8 @@ void RestoreFileInfo(StkPtr)
    rcbRadius = StkPtr->rcb_radius;
    curUnderlineOn = StkPtr->underline_on;
    curUnderlineYOffset = StkPtr->underline_y_offset;
+   curOverlineOn = StkPtr->overline_on;
+   curOverlineYOffset = StkPtr->overline_y_offset;
    tgifObj->fattr = StkPtr->first_file_attr;
    tgifObj->lattr = StkPtr->last_file_attr;
 
