@@ -2,7 +2,7 @@
  * Author:      Daniel Ratton Figueiredo <ratton@land.ufrj.br>
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -16,7 +16,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.8 2006/05/30 02:59:32 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.19 2011/05/25 16:45:49 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -29,6 +29,7 @@
 #include "auxtext.e"
 #include "box.e"
 #include "cmd.e"
+#include "color.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "exec.e"
@@ -48,8 +49,6 @@
 
 char	cmdLineGenParser[MAXSTRING];
 
-#ifdef _TANGRAM_II
-
 #define DO_CMD(cmd) ExecACommandFromBuffer(cmd,NULL)
 
 static TgMenuItemInfo tangram2MenuItemInfo[] = {
@@ -244,7 +243,7 @@ static struct ObjRec *objSp=NULL; /* Simulation Step  button */
 #define MAXFORMATSIZE           24
 #define MAXEVENTNAME            512
 
-#define SKIP_WHITESPACE(x) { while( isspace(*(x)) ) (x)++; }
+#define SKIP_WHITESPACE(x) { while( isspace((int)(*(x))) ) (x)++; }
 
 /* Control Packet */
 typedef struct tagT_ControlPacket {
@@ -412,7 +411,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     char *service = "6743";
 
     strcpy( host, "localhost" );
-    bzero( (char *)&sin, sizeof( sin ) );
+    memset( (char *)&sin, 0, sizeof( sin ) );
     sin.sin_family = AF_INET;
     if( (pse = getservbyname( service, protocol )) != NULL )
         sin.sin_port = pse->s_port;
@@ -488,7 +487,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     {
         sin_size = sizeof( sin );
         if( recvfrom( s, &buffer, sizeof(buffer), 0, (struct sockaddr *)&sin,
-                      (socklen_t *)(&sin_size) ) < 0 )
+                      (void*)(&sin_size) ) < 0 )
         {
             perror( "TGIF: recv" );
             return;
@@ -793,13 +792,13 @@ int * options; int * initial_index; int * final_index;
     /* Syntax for requests: symname([i(:f)])(<symformat>)=(symvalue) */
 
     SKIP_WHITESPACE(buffer);
-    if( !isalpha(*buffer) ) return -1;
+    if( !isalpha((int)(*buffer)) ) return -1;
 
     *options = 0;
 
     /* getting the variable name */
-    for( i = 0; isalpha(*buffer) ||
-                isdigit(*buffer) ||
+    for( i = 0; isalpha((int)(*buffer)) ||
+                isdigit((int)(*buffer)) ||
                 *buffer == '_';
          buffer++, i++ )
     {
@@ -1290,7 +1289,7 @@ static char *GenerateChainExec[] = {
 };
 
 static char *GenerateChainWriteModel[] = {
-  "find_obj_names(obj_list,\"\",\"Declaration=*\");",
+  "find_obj_names_on_all_pages(obj_list,\"Declaration=*\");",
   "get_line_in_attr(total_objs,obj_list,0);",
   "for_i(i,1,$(total_objs),1,write_obj);",
   NULL
@@ -1594,6 +1593,10 @@ char *mode;
    tmp_box_obj = topObj;
 
    tmp_box_obj->color = 9;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(tmp_box_obj->color_str, sizeof(tmp_box_obj->color_str),
+            colorMenuItems[tmp_box_obj->color]);
+   }
    tmp_box_obj->detail.rcb->width = 2;
    y += 5;
 
@@ -1746,7 +1749,7 @@ int max_arg_length;
     int i, j;
     char *ptr;
 
-    bzero( dst, no_args * max_arg_length );    
+    memset( dst, 0, no_args * max_arg_length );    
     ptr = args;
     /* Remove leading " and spaces */
     while( *ptr == '\"' || *ptr == ' ' )
@@ -2037,4 +2040,3 @@ void Tangram2UpdateInitAttr(port_obj, signal_name)
    UpdTextBBox(attr_ptr->obj);
 }
 
-#endif /* _TANGRAM_II */
