@@ -29,7 +29,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.18 2011/05/16 16:21:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.19 2011/05/25 16:45:49 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -256,7 +256,7 @@ static struct ObjRec *objSp=NULL; /* Simulation Step  button */
 #define MAXFORMATSIZE           24
 #define MAXEVENTNAME            512
 
-#define SKIP_WHITESPACE(x) { while( isspace(*(x)) ) (x)++; }
+#define SKIP_WHITESPACE(x) { while( isspace((int)(*(x))) ) (x)++; }
 
 /* Control Packet */
 typedef struct tagT_ControlPacket {
@@ -805,13 +805,13 @@ int * options; int * initial_index; int * final_index;
     /* Syntax for requests: symname([i(:f)])(<symformat>)=(symvalue) */
 
     SKIP_WHITESPACE(buffer);
-    if( !isalpha(*buffer) ) return -1;
+    if( !isalpha((int)(*buffer)) ) return -1;
 
     *options = 0;
 
     /* getting the variable name */
-    for( i = 0; isalpha(*buffer) ||
-                isdigit(*buffer) ||
+    for( i = 0; isalpha((int)(*buffer)) ||
+                isdigit((int)(*buffer)) ||
                 *buffer == '_';
          buffer++, i++ )
     {
