@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtbtn.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtbtn.c,v 1.8 2011/05/16 16:22:00 william Exp $
  */
 
 #define _INCLUDE_FROM_TDGTBTN_C_
@@ -129,7 +129,7 @@ void RedrawTdgtBtn(pti)
       break;
    case TGMUTYPE_COLOR:
       if (colorDisplay) {
-         pixel = (btn_down ? myFgPixel : colorPixels[(int)(pTdgtBtn->str)]);
+         pixel = (btn_down ? myFgPixel : colorPixels[(int)(long)(pTdgtBtn->str)]);
       } else {
          pixel = myFgPixel;
       }
@@ -508,6 +508,9 @@ TdgtBtn *CreateTdgtBtn(parent_win, parent_tidgetinfo, ctl_id, x, y, w, h, h_pad,
          x, y, w, h, brdrW, myBorderPixel, bg_pixel)) == 0) {
       FailToCreateWindowMessage("CreateTdgtBtn()", NULL, TRUE);
    }
+   XSelectInput(mainDisplay, pTdgtBtn->pti->tci.win,
+         ButtonReleaseMask | ButtonPressMask | KeyPressMask | ExposureMask |
+         EnterWindowMask | LeaveWindowMask);
    SetTidgetInfoBasic(pTdgtBtn->pti, TIDGET_TYPE_BTN, pTdgtBtn, parent_win,
          x, y, w, h, h_pad, v_pad, state, NULL);
    TidgetSetCallbacks(pTdgtBtn->pti,
