@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtbtn.c,v 1.6 2009/01/28 20:39:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtbtn.c,v 1.8 2011/05/16 16:22:00 william Exp $
  */
 
 #define _INCLUDE_FROM_TDGTBTN_C_
@@ -142,7 +142,7 @@ void RedrawTdgtBtn(pti)
       break;
    case TGMUTYPE_COLOR:
       if (colorDisplay) {
-         pixel = (btn_down ? myFgPixel : colorPixels[(int)(pTdgtBtn->str)]);
+         pixel = (btn_down ? myFgPixel : colorPixels[(int)(long)(pTdgtBtn->str)]);
       } else {
          pixel = myFgPixel;
       }
