@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtmsg.c,v 1.7 2009/06/09 17:36:40 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtmsg.c,v 1.9 2011/05/16 16:22:00 william Exp $
  */
 
 #define _INCLUDE_FROM_TDGTMSG_C_
@@ -75,7 +75,9 @@ void TdgtMsgDrawCaption(pTdgtMsg)
    char *psz=NULL, *psz_cur=TdgtMsgGetText(pTdgtMsg);
 
    if (boldMsgFontSet != NULL || boldMsgFontPtr != NULL) {
-       XSetFont(mainDisplay, gTidgetManager.gc, boldMsgFontPtr->fid);
+       if (boldMsgFontPtr != NULL) {
+          XSetFont(mainDisplay, gTidgetManager.gc, boldMsgFontPtr->fid);
+       }
        y += boldMsgFontAsc;
    } else if (msgFontSet != NULL || msgFontPtr != NULL) {
        XSetFont(mainDisplay, gTidgetManager.gc, msgFontPtr->fid);
