@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtmsg.c,v 1.2 2005/01/15 00:23:23 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtmsg.c,v 1.9 2011/05/16 16:22:00 william Exp $
  */
 
 #define _INCLUDE_FROM_TDGTMSG_C_
@@ -61,10 +61,16 @@ void TdgtMsgDrawCaption(pTdgtMsg)
    int win_w=pTdgtMsg->pti->tci.win_info.w;
    char *psz=NULL, *psz_cur=TdgtMsgGetText(pTdgtMsg);
 
-   if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
-       y += defaultFontAsc;
-   } else {
+   if (boldMsgFontSet != NULL || boldMsgFontPtr != NULL) {
+       if (boldMsgFontPtr != NULL) {
+          XSetFont(mainDisplay, gTidgetManager.gc, boldMsgFontPtr->fid);
+       }
        y += boldMsgFontAsc;
+   } else if (msgFontSet != NULL || msgFontPtr != NULL) {
+       XSetFont(mainDisplay, gTidgetManager.gc, msgFontPtr->fid);
+       y += msgFontAsc;
+   } else {
+       y += defaultFontAsc;
    }
    switch (just) {
    case JUST_L: break;
@@ -93,10 +99,12 @@ void TdgtMsgDrawCaption(pTdgtMsg)
                gTidgetManager.gc, x-line_w, y, psz_cur, strlen(psz_cur));
          break;
       }
-      if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
-          y += defaultFontHeight;
-      } else {
+      if (boldMsgFontSet != NULL || boldMsgFontPtr != NULL) {
           y += boldMsgFontHeight;
+      } else if (msgFontSet != NULL || msgFontPtr != NULL) {
+          y += msgFontHeight;
+      } else {
+          y += defaultFontHeight;
       }
       if (psz != NULL) {
          *psz++ = '\n';
@@ -270,10 +278,12 @@ void TdgtMsgCalcGeom(caption, pn_w, pn_h)
       if (psz != NULL) *psz = '\0';
       line_w = BoldMsgTextWidth(boldMsgFontPtr, psz_cur, strlen(psz_cur));
       if (line_w > w) w = line_w;
-      if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
-          h += defaultFontHeight;
-      } else {
+      if (boldMsgFontSet != NULL || boldMsgFontPtr != NULL) {
           h += boldMsgFontHeight;
+      } else if (msgFontSet != NULL || msgFontPtr != NULL) {
+          h += msgFontHeight;
+      } else {
+          h += defaultFontHeight;
       }
       if (psz != NULL) {
          *psz++ = '\n';
@@ -317,6 +327,9 @@ TdgtMsg *CreateTdgtMsg(parent_win, parent_tidgetinfo, ctl_id, x, y, h_pad,
       FailToCreateWindowMessage("CreateTdgtMsg()", NULL, TRUE);
       return NULL;
    }
+   XSelectInput(mainDisplay, pTdgtMsg->pti->tci.win,
+         ButtonReleaseMask | ButtonPressMask | KeyPressMask | ExposureMask |
+         EnterWindowMask | LeaveWindowMask);
    SetTidgetInfoBasic(pTdgtMsg->pti, TIDGET_TYPE_MSG, pTdgtMsg, parent_win,
          x, y, w, h, h_pad, v_pad, state, caption);
    TidgetSetCallbacks(pTdgtMsg->pti,
