@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgcwheel.c,v 1.4 2011/05/16 16:22:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgcwheel.c,v 1.5 2011/06/02 15:53:08 william Exp $
  */
 
 #define _INCLUDE_FROM_TGCWHEEL__C_
@@ -105,7 +105,7 @@ int TgColorWheel_Init(dpy, win, arg)
 
 void CleanUpColorWheel()
 {
-   return TgColorWheel_CleanUp(mainDisplay, drawWindow);
+   TgColorWheel_CleanUp(mainDisplay, drawWindow);
 }
 
 int InitColorWheel()
