@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tginssym.c,v 1.4 2011/05/16 16:22:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tginssym.c,v 1.5 2011/06/02 15:53:08 william Exp $
  */
 
 #define _INCLUDE_FROM_TGINSSYM__C_
@@ -131,7 +131,7 @@ int TgInsertSymbol_Init(dpy, win, arg)
 
 void CleanUpInsertSymbol()
 {
-   return TgInsertSymbol_CleanUp(mainDisplay, drawWindow);
+   TgInsertSymbol_CleanUp(mainDisplay, drawWindow);
 }
 
 int InitInsertSymbol()
