@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgtwb5.c,v 1.7 2009/04/11 23:18:21 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgtwb5.c,v 1.11 2011/05/16 16:22:00 william Exp $
  */
 
 #ifndef _NO_TGTWB5
@@ -82,9 +82,15 @@ int Tgtwb5_Init(dpy, win, arg)
       gZyfhInfo.b5_font_name = UtilStrDup(szDefB5FontName);
       if (gZyfhInfo.b5_font_name == NULL) FailAllocMessage();
    }
-   gZyfhInfo.xfs = LoadAFontByName(gZyfhInfo.b5_font_name, TRUE, TRUE);
-   if (gZyfhInfo.xfs == NULL) {
-      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_LOAD_TGTWB5_FONT),
+   if (strstr(gZyfhInfo.b5_font_name, "-gb2312") != NULL) {
+      gZyfhInfo.b5_font_real_encoding = GB_ENCODING;
+      UtilStrCpyN(gZyfhInfo.im_name, sizeof(gZyfhInfo.im_name), "tgchgb");
+   } else if (strstr(gZyfhInfo.b5_font_name, "-big5") != NULL) {
+      gZyfhInfo.b5_font_real_encoding = BIG5_ENCODING;
+      UtilStrCpyN(gZyfhInfo.im_name, sizeof(gZyfhInfo.im_name), "tgtwb5");
+   } else {
+      gZyfhInfo.b5_font_real_encoding = INVALID;
+      sprintf(gszMsgBox, TgLoadString(STID_BAD_ENCODING_TGTWB5_FONT),
             gZyfhInfo.b5_font_name, TOOL_NAME, "DoubleByteInputMethod");
       fprintf(stderr, "%s\n", gszMsgBox);
 
@@ -92,6 +98,18 @@ int Tgtwb5_Init(dpy, win, arg)
 
       return FALSE;
    }
+   gZyfhInfo.xfs = LoadAFontByName(gZyfhInfo.b5_font_name, TRUE, TRUE);
+   if (gZyfhInfo.xfs == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_LOAD_TGTWB5_FONT),
+            gZyfhInfo.b5_font_name, TOOL_NAME, "DoubleByteInputMethod",
+            gZyfhInfo.im_name);
+      fprintf(stderr, "%s\n", gszMsgBox);
+
+      Tgtwb5_CleanUp(dpy, win);
+
+      return FALSE;
+   }
+   gZyfhInfo.b5_font_mod_bytes = ((gZyfhInfo.xfs->min_byte1 & 0x80) == 0);
    gZyfhInfo.b5_font_w = gZyfhInfo.xfs->max_bounds.rbearing;
    gZyfhInfo.b5_font_asc = gZyfhInfo.xfs->max_bounds.ascent;
    gZyfhInfo.b5_font_des = gZyfhInfo.xfs->max_bounds.descent;
@@ -104,10 +122,10 @@ int Tgtwb5_Init(dpy, win, arg)
    gZyfhInfo.zyfh_bitmap_h = tgtwb5_height;
    gZyfhInfo.zyfh_image_wh =
          round(((double)tgtwb5_width)/((double)gZyfhInfo.zyfh_num_cols));
-   gZyfhInfo.zyfh_bitmap = XCreateBitmapFromData (mainDisplay, mainWindow,
+   gZyfhInfo.zyfh_bitmap = XCreateBitmapFromData(mainDisplay, mainWindow,
            (char*)tgtwb5_bits, tgtwb5_width, tgtwb5_height);
 
-   gZyfhInfo.symbol_bitmap = XCreatePixmap (mainDisplay, rootWindow,
+   gZyfhInfo.symbol_bitmap = XCreatePixmap(mainDisplay, rootWindow,
          gZyfhInfo.zyfh_image_wh, gZyfhInfo.zyfh_image_wh, 1);
 
    if (!InitTdgtb5Dlg()) {
