@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgtwb5dl.c,v 1.8 2009/06/08 21:42:38 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgtwb5dl.c,v 1.15 2011/05/22 22:55:02 william Exp $
  */
 
 #ifndef _NO_TGTWB5
@@ -228,6 +228,74 @@ void FreeBitmapItemCallback(pblii)
    free(pblii);
 }
 
+#ifdef NOT_DEFINED /* debug, do not translate */
+static
+void DebugCharBitmap(bitmap, w, h)
+   Pixmap bitmap;
+   int w, h;
+{
+   static int n=0;
+   int r=0, c=0;
+   XImage *image=XGetImage(mainDisplay, bitmap, 0, 0, w,
+         h, 1, ZPixmap);
+
+   fprintf(stdout, "Character %1d:\n", ++n);
+   for (r=0; r < h; r++) {
+      fprintf(stdout, "    ");
+      for (c=0; c < w; c++) {
+         if (XGetPixel(image, (int)c, (int)r) == 1) {
+            fprintf(stdout, "%1d", 1);
+         } else {
+            fprintf(stdout, "%1d", 0);
+         }
+      }
+      fprintf(stdout, "\n");
+   }
+   XDestroyImage(image);
+}
+
+void DrawAndDebugChar()
+{
+   XChar2b str16;
+   XGCValues values;
+   GC tmp_gc;
+   Pixmap bitmap=XCreatePixmap(mainDisplay, mainWindow, gZyfhInfo.b5_font_w,
+         gZyfhInfo.b5_font_h, 1);
+
+   values.foreground = 1;
+   values.background = 0;
+   values.fill_style = FillSolid;
+   values.function = GXcopy;
+   values.font = gZyfhInfo.xfs->fid;
+   tmp_gc = XCreateGC(mainDisplay, bitmap,
+         GCForeground | GCBackground | GCFillStyle | GCFunction | GCFont,
+         &values);
+
+   XSetForeground(mainDisplay, tmp_gc, 0);
+   XFillRectangle(mainDisplay, bitmap, tmp_gc, 0, 0, gZyfhInfo.b5_font_w,
+         gZyfhInfo.b5_font_h);
+   XSetForeground(mainDisplay, tmp_gc, 1);
+
+   switch (gZyfhInfo.b5_font_real_encoding) {
+   case GB_ENCODING:
+      str16.byte1 = 0xb9 & 0x7f;
+      str16.byte2 = 0xfa & 0x7f;
+      break;
+   case BIG5_ENCODING:
+      str16.byte1 = 0xa4;
+      str16.byte2 = 0xad;
+      break;
+   }
+   XDrawString16(mainDisplay, bitmap, tmp_gc, 0, gZyfhInfo.b5_font_asc,
+         &str16, 1);
+
+   DebugCharBitmap(bitmap, gZyfhInfo.b5_font_w, gZyfhInfo.b5_font_h);
+
+   XFreeGC(mainDisplay, tmp_gc);
+   XFreePixmap(mainDisplay, bitmap);
+}
+#endif /* NOT_DEFINED */
+
 static
 int DoLoadChars(pTdgtBase, buf, str16, num_char2bs)
    TdgtBase *pTdgtBase;
@@ -246,6 +314,7 @@ int DoLoadChars(pTdgtBase, buf, str16, num_char2bs)
       Pixmap bitmap=None;
       BmpListItemInfo *pblii=NULL;
       XChar2b *pch16=NULL;
+      XChar2b str16_buf;
 
       bitmap = XCreatePixmap(mainDisplay, mainWindow, gZyfhInfo.b5_font_w,
             gZyfhInfo.b5_font_h, 1);
@@ -260,8 +329,19 @@ int DoLoadChars(pTdgtBase, buf, str16, num_char2bs)
       values.font = gZyfhInfo.xfs->fid;
       XChangeGC(mainDisplay, rotateGC, GCForeground | GCFont, &values);
 
+      if (gZyfhInfo.b5_font_mod_bytes) {
+         str16_buf.byte1 = str16[i].byte1 & 0x7f;
+         str16_buf.byte2 = str16[i].byte2 & 0x7f;
+      } else {
+         str16_buf.byte1 = str16[i].byte1;
+         str16_buf.byte2 = str16[i].byte2;
+      }
       XDrawString16(mainDisplay, bitmap, rotateGC, 0, gZyfhInfo.b5_font_asc,
-            &str16[i], 1);
+            &str16_buf, 1);
+
+#ifdef NOT_DEFINED
+      DebugCharBitmap(bitmap, gZyfhInfo.b5_font_w, gZyfhInfo.b5_font_h);
+#endif /* NOT_DEFINED */
 
       pch16 = (XChar2b*)malloc(sizeof(XChar2b));
       if (pch16 == NULL) FailAllocMessage();
@@ -287,7 +367,7 @@ int DoLoadChars(pTdgtBase, buf, str16, num_char2bs)
    return TRUE;
 }
 
-#define MAX_PINYIN_ET_BIG5_STR 576
+#define MAX_PINYIN_ET_BIG5_STR 628
 #define MAX_BIG5_STR (MAX_PINYIN_ET_BIG5_STR>>2)
 
 static
@@ -305,12 +385,26 @@ int LoadChars(pTdgtBase, buf)
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
-   for (ppsz=pinyin_et_big5; *ppsz!=NULL; ppsz++) {
-      if (strcmp(*ppsz, buf) == 0) {
-         psz_found = (*(++ppsz));
-         break;
+   switch (gZyfhInfo.b5_font_real_encoding) {
+   case GB_ENCODING:
+      for (ppsz=pinyin_gb2312; *ppsz!=NULL; ppsz++) {
+         if (strcmp(*ppsz, buf) == 0) {
+            psz_found = (*(++ppsz));
+            break;
+         }
+         ppsz++;
       }
-      ppsz++;
+      break;
+   case BIG5_ENCODING:
+      for (ppsz=pinyin_et_big5; *ppsz!=NULL; ppsz++) {
+         if (strcmp(*ppsz, buf) == 0) {
+            psz_found = (*(++ppsz));
+            break;
+         }
+         ppsz++;
+      }
+      break;
+   default: return FALSE;
    }
    if (psz_found == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_INVALID_PINYIN_SEQ), buf);
@@ -337,7 +431,14 @@ int LoadChars(pTdgtBase, buf)
 
 #ifdef _TGIF_DBG /* debug, do not translate */
    if (!ok) {
-      sprintf(gszMsgBox, "The pinyin_et_big5[] table seems to be corrupted.\n");
+      switch (gZyfhInfo.b5_font_real_encoding) {
+      case GB_ENCODING:
+         sprintf(gszMsgBox, "The pinyin_gb2312[] table seems to be corrupted.\n");
+         break;
+      case BIG5_ENCODING:
+         sprintf(gszMsgBox, "The pinyin_et_big5[] table seems to be corrupted.\n");
+         break;
+      }
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
@@ -1174,7 +1275,15 @@ TidgetInfo *CreateTdgtb5DialogBox(dpy, parent_win)
    x = h_pad+windowPadding;
    y = v_pad+windowPadding;
 
-   sprintf(caption, TgLoadString(STID_TOOL_CHINESE_KEYPAD), TOOL_NAME);
+   switch (gZyfhInfo.b5_font_real_encoding) {
+   case GB_ENCODING:
+      sprintf(caption, TgLoadString(STID_TOOL_GB_CHINESE_KEYPAD), TOOL_NAME);
+      break;
+   case BIG5_ENCODING:
+      sprintf(caption, TgLoadString(STID_TOOL_BIG5_CHINESE_KEYPAD), TOOL_NAME);
+      break;
+   default: return NULL;
+   }
    pTdgtBase = CreateTdgtBase(parent_win, NULL, ID_ZYFH_DIALOG, 0, 0, 100, 100,
          h_pad, v_pad, (dialogboxUse3DBorder ? TGBS_RAISED : TGBS_NORMAL),
          caption);
@@ -1347,6 +1456,9 @@ int Tdgtb5DlgLoop(pti)
          }
       } else if (xev.type == Expose) {
          ExposeEventHandler(&xev, FALSE);
+      } else if (xev.type == ConfigureNotify &&
+            xev.xany.window==mainWindow) {
+         Reconfigure(FALSE);
       } else if (xev.type == KeyPress) {
          TidgetManagerHandleAllKeyPressEvent(&xev);
       } else if (IsTdgtNotifyEvent(pti, &xev, &tdgt_notify)) {
