@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2009, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2011, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/types.h,v 1.42 2009/09/30 14:53:55 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/types.h,v 1.47 2011/05/16 16:22:00 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -1059,4 +1059,25 @@ typedef struct tagGeneratedByInfoRec {
    char version_str[40]; /* version of the tool */
 } GenerateByInfo;
 
+/* ------------------ pngtrans stuff ------------------ */
+
+typedef struct tagPngHeaderInfo {
+    int valid;
+    char fname[MAXPATHLENGTH];
+    FILE *fp;
+    uint32_t image_w;
+    uint32_t image_h;
+    unsigned char bit_depth;
+    unsigned char color_type;
+    unsigned char compression_method;
+    unsigned char filter_method;
+    unsigned char interlace_method;
+    uint32_t palette_sz;
+    unsigned char *palette;
+    int trans_color_pixel_found;
+    unsigned char trans_color_pixel_red;
+    unsigned char trans_color_pixel_green;
+    unsigned char trans_color_pixel_blue;
+} PngHeaderInfo;
+
 #endif /*_TGIF_TYPES_H_*/
