@@ -3,15 +3,11 @@
  *
  * (Obsoleted in tgif-4.1.41 by the TGWB group at land.ufrj.br.)
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/wb_mcast.c,v 1.1 2004/06/18 23:19:10 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/wb_mcast.c,v 1.2 2008/10/16 21:13:03 william Exp $
  */
 
 #define _INCLUDE_FROM_WB_MCAST_C_
 
 #include "tgifdefs.h"
 
-#ifdef _TGIF_WB2
-
 #include "wb_mcast.e"
-
-#endif /* _TGIF_WB2 */
