@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/z_intrf.c,v 1.10 2011/05/16 16:22:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/z_intrf.c,v 1.11 2011/06/18 04:44:51 william Exp $
  */
 
 #define _INCLUDE_FROM_Z_INTRF_C_
@@ -52,7 +52,7 @@
 static
 void ByteStuff(buf_in, buf_len, buf_out, pn_buf_out)
    unsigned char *buf_in, *buf_out;
-   int buf_len, *pn_buf_out;
+   unsigned int buf_len, *pn_buf_out;
 {
    int i=0, count=0;
 
